/*
    Texel - A UCI chess engine.
    Copyright (C) 2012,2014  Peter Österlund, peterosterlund2@gmail.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/*
 * krkpTable.cpp
 *
 *  Created on: Feb 26, 2012
 *      Author: petero
 */

#include "endGameEval.hpp"

const U8 EndGameEval::krkpTable[2*32*48*8] = {
  0x00, 0x0c, 0x0f, 0x0f, 0x07, 0x00, 0x00, 0x00, 0x08, 0x0c, 0x06, 0x06,
  0x00, 0x00, 0x00, 0x00, 0x1c, 0x18, 0x1f, 0x00, 0x00, 0x00, 0x00, 0x00,
  0xfc, 0xf4, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xfc, 0xec, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xfc, 0xdc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfc, 0xbc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0x7c, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xff, 0x1c, 0x1c, 0x1e, 0x0f, 0x0f, 0x0f, 0x00, 0x00,
  0x1c, 0x1c, 0x1d, 0x1f, 0x0f, 0x00, 0x00, 0x00, 0x3c, 0x3c, 0x3b, 0x3f,
  0x00, 0x00, 0x00, 0x00, 0xfc, 0xfc, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfc, 0xfc, 0xef, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xdf, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xbf, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfc, 0xfc, 0x7f, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xff, 0xfe,
  0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xff,
  0xfc, 0xfc, 0xff, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xff, 0xf7,
  0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xff, 0xef, 0xff, 0xff, 0xff, 0xff,
  0xfc, 0xfc, 0xff, 0xdf, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xff, 0xbf,
  0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xff, 0x7f, 0xff, 0xff, 0xff, 0xff,
  0xfc, 0xfc, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xff, 0xff,
  0xfd, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xff,
  0xfc, 0xfc, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xff, 0xff,
  0xef, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xff, 0xff, 0xdf, 0xff, 0xff, 0xff,
  0xfc, 0xfc, 0xff, 0xff, 0xbf, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xff, 0xff,
  0x7f, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xfe, 0xff, 0xff,
  0xfc, 0xfc, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xfc, 0xfc, 0xff, 0xff,
  0xff, 0xfb, 0xff, 0xff, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xff,
  0xfc, 0xfc, 0xff, 0xff, 0xff, 0xef, 0xff, 0xff, 0xfc, 0xfc, 0xff, 0xff,
  0xff, 0xdf, 0xff, 0xff, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xbf, 0xff, 0xff,
  0xfc, 0xfc, 0xff, 0xff, 0xff, 0x7f, 0xff, 0xff, 0xfc, 0xfc, 0xff, 0xff,
  0xff, 0xff, 0xfe, 0xff, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xff,
  0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xfc, 0xfc, 0xff, 0xff,
  0xff, 0xff, 0xf7, 0xff, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xef, 0xff,
  0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xdf, 0xff, 0xfc, 0xfc, 0xff, 0xff,
  0xff, 0xff, 0xbf, 0xff, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0x7f, 0xff,
  0x00, 0x00, 0x07, 0x07, 0x00, 0x00, 0x00, 0x00, 0x18, 0x18, 0x1f, 0x0f,
  0x00, 0x00, 0x00, 0x00, 0x10, 0x18, 0x0c, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x38, 0x30, 0x3e, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf8, 0xe8, 0xff, 0xff,
  0xef, 0xff, 0xff, 0xff, 0xf8, 0xd8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xf8, 0xb8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0x78, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xff, 0x08, 0x08, 0x0e, 0x0f, 0x0f, 0x00, 0x00, 0x00,
  0x38, 0x38, 0x3d, 0x1f, 0x1f, 0x00, 0x00, 0x00, 0x38, 0x38, 0x3b, 0x3e,
  0x00, 0x00, 0x00, 0x00, 0x78, 0x78, 0x77, 0x7f, 0x00, 0x00, 0x00, 0x00,
  0xf8, 0xf8, 0xef, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xdf, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xbf, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xf8, 0xf8, 0x7f, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xff, 0xfe,
  0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xff,
  0xf8, 0xf8, 0xff, 0xfb, 0xff, 0xfb, 0xff, 0xff, 0xf8, 0xf8, 0xff, 0xf7,
  0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xff, 0xef, 0xff, 0xff, 0xff, 0xff,
  0xf8, 0xf8, 0xff, 0xdf, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xff, 0xbf,
  0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xff, 0x7f, 0xff, 0xff, 0xff, 0xff,
  0xf8, 0xf8, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xff, 0xff,
  0xfd, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xff,
  0xf8, 0xf8, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xff, 0xff,
  0xef, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xff, 0xff, 0xdf, 0xff, 0xff, 0xff,
  0xf8, 0xf8, 0xff, 0xff, 0xbf, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xff, 0xff,
  0x7f, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xfe, 0xff, 0xff,
  0xf8, 0xf8, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xf8, 0xf8, 0xff, 0xff,
  0xff, 0xfb, 0xff, 0xff, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xff,
  0xf8, 0xf8, 0xff, 0xff, 0xff, 0xef, 0xff, 0xff, 0xf8, 0xf8, 0xff, 0xff,
  0xff, 0xdf, 0xff, 0xff, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xbf, 0xff, 0xff,
  0xf8, 0xf8, 0xff, 0xff, 0xff, 0x7f, 0xff, 0xff, 0xf8, 0xf8, 0xff, 0xff,
  0xff, 0xff, 0xfe, 0xff, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xff,
  0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xf8, 0xf8, 0xff, 0xff,
  0xff, 0xff, 0xf7, 0xff, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xef, 0xff,
  0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xdf, 0xff, 0xf8, 0xf8, 0xff, 0xff,
  0xff, 0xff, 0xbf, 0xff, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0x7f, 0xff,
  0x01, 0x00, 0x07, 0x07, 0x00, 0x00, 0x00, 0x00, 0x10, 0x11, 0x0f, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x31, 0x31, 0x3f, 0x1e, 0x00, 0x00, 0x00, 0x00,
  0x21, 0x31, 0x18, 0x00, 0x00, 0x00, 0x00, 0x00, 0x70, 0x60, 0x7c, 0x00,
  0x00, 0x00, 0x00, 0x00, 0xf1, 0xd1, 0xff, 0xff, 0xdf, 0xff, 0xff, 0xff,
  0xf1, 0xb1, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf1, 0x71, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xff, 0x01, 0x01, 0x0e, 0x0f, 0x0f, 0x00, 0x00, 0x00,
  0x31, 0x31, 0x3d, 0x1f, 0x00, 0x00, 0x00, 0x00, 0x71, 0x71, 0x7b, 0x3f,
  0x3e, 0x00, 0x00, 0x00, 0x71, 0x71, 0x77, 0x7c, 0x00, 0x00, 0x00, 0x00,
  0xf0, 0xf0, 0xee, 0xfe, 0x00, 0x00, 0x00, 0x00, 0xf1, 0xf1, 0xdf, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xbf, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xf1, 0xf1, 0x7f, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xff, 0xfe,
  0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xff, 0xfd, 0xff, 0xfd, 0xff, 0xff,
  0xf1, 0xf1, 0xff, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xff, 0xf7,
  0xff, 0xf7, 0xff, 0xff, 0xf1, 0xf1, 0xff, 0xef, 0xff, 0xff, 0xff, 0xff,
  0xf1, 0xf1, 0xff, 0xdf, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xff, 0xbf,
  0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xff, 0x7f, 0xff, 0xff, 0xff, 0xff,
  0xf1, 0xf1, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xff, 0xff,
  0xfd, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xff,
  0xf1, 0xf1, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xff, 0xff,
  0xef, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xff, 0xff, 0xdf, 0xff, 0xff, 0xff,
  0xf1, 0xf1, 0xff, 0xff, 0xbf, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xff, 0xff,
  0x7f, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xfe, 0xff, 0xff,
  0xf1, 0xf1, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xf1, 0xf1, 0xff, 0xff,
  0xff, 0xfb, 0xff, 0xff, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xff,
  0xf1, 0xf1, 0xff, 0xff, 0xff, 0xef, 0xff, 0xff, 0xf1, 0xf1, 0xff, 0xff,
  0xff, 0xdf, 0xff, 0xff, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xbf, 0xff, 0xff,
  0xf1, 0xf1, 0xff, 0xff, 0xff, 0x7f, 0xff, 0xff, 0xf1, 0xf1, 0xff, 0xff,
  0xff, 0xff, 0xfe, 0xff, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xff,
  0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xf1, 0xf1, 0xff, 0xff,
  0xff, 0xff, 0xf7, 0xff, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xef, 0xff,
  0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xdf, 0xff, 0xf1, 0xf1, 0xff, 0xff,
  0xff, 0xff, 0xbf, 0xff, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0x7f, 0xff,
  0xe3, 0xe2, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x03, 0x01, 0x0f, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x21, 0x23, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x63, 0x63, 0x7f, 0x1c, 0x00, 0x00, 0x00, 0x00, 0x42, 0x62, 0x38, 0x00,
  0x00, 0x00, 0x00, 0x00, 0xe0, 0xc0, 0xf8, 0x00, 0x00, 0x00, 0x00, 0x00,
  0xe3, 0xa3, 0xff, 0xff, 0xbf, 0xff, 0xff, 0xff, 0xe3, 0x63, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff,
  0x03, 0x03, 0x1d, 0x1f, 0x00, 0x00, 0x00, 0x00, 0x63, 0x63, 0x7b, 0x3f,
  0x00, 0x00, 0x00, 0x00, 0xe3, 0xe3, 0xf7, 0x7f, 0x7c, 0x00, 0x00, 0x00,
  0xe3, 0xe3, 0xef, 0xfe, 0x00, 0x00, 0x00, 0x00, 0xe0, 0xe0, 0xdc, 0xfc,
  0x00, 0x00, 0x00, 0x00, 0xe3, 0xe3, 0xbf, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xe3, 0xe3, 0x7f, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xff, 0xfe,
  0xff, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xff,
  0xe3, 0xe3, 0xff, 0xfb, 0xff, 0xfb, 0xff, 0xff, 0xe3, 0xe3, 0xff, 0xf7,
  0xff, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xff, 0xef, 0xff, 0xef, 0xff, 0xff,
  0xe3, 0xe3, 0xff, 0xdf, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xff, 0xbf,
  0xff, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xff, 0x7f, 0xff, 0xff, 0xff, 0xff,
  0xe3, 0xe3, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xff, 0xff,
  0xfd, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xff,
  0xe3, 0xe3, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xff, 0xff,
  0xef, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xff, 0xff, 0xdf, 0xff, 0xff, 0xff,
  0xe3, 0xe3, 0xff, 0xff, 0xbf, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xff, 0xff,
  0x7f, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xfe, 0xff, 0xff,
  0xe3, 0xe3, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xe3, 0xe3, 0xff, 0xff,
  0xff, 0xfb, 0xff, 0xff, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xff,
  0xe3, 0xe3, 0xff, 0xff, 0xff, 0xef, 0xff, 0xff, 0xe3, 0xe3, 0xff, 0xff,
  0xff, 0xdf, 0xff, 0xff, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xbf, 0xff, 0xff,
  0xe3, 0xe3, 0xff, 0xff, 0xff, 0x7f, 0xff, 0xff, 0xe3, 0xe3, 0xff, 0xff,
  0xff, 0xff, 0xfe, 0xff, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xff,
  0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xe3, 0xe3, 0xff, 0xff,
  0xff, 0xff, 0xf7, 0xff, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xef, 0xff,
  0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xdf, 0xff, 0xe3, 0xe3, 0xff, 0xff,
  0xff, 0xff, 0xbf, 0xff, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff, 0x7f, 0xff,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x08, 0x0c, 0x04, 0x06,
  0x00, 0x00, 0x00, 0x00, 0x1c, 0x18, 0x1c, 0x00, 0x00, 0x00, 0x00, 0x00,
  0xfc, 0xf4, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xec, 0xfc, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xfc, 0xdc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfc, 0xbc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0x7c, 0xfc, 0xff,
  0xff, 0xff, 0xff, 0xff, 0x1c, 0x1c, 0x1c, 0x0f, 0x0f, 0x0f, 0x00, 0x00,
  0x1c, 0x1c, 0x1c, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x3c, 0x3c, 0x38, 0x3f,
  0x00, 0x00, 0x00, 0x00, 0xfc, 0xfc, 0xf4, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfc, 0xfc, 0xec, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xdc, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xbc, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfc, 0xfc, 0x7c, 0xff, 0xff, 0xff, 0xff, 0xff, 0x3c, 0x3c, 0x3c, 0x3e,
  0x1f, 0x1f, 0x1f, 0x00, 0x7c, 0x7c, 0x7c, 0x7d, 0x3f, 0x1f, 0x00, 0x00,
  0xfc, 0xfc, 0xfc, 0xfb, 0xff, 0xfe, 0x00, 0x00, 0xfc, 0xfc, 0xfc, 0xf7,
  0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xef, 0xff, 0xff, 0xff, 0xff,
  0xfc, 0xfc, 0xfc, 0xdf, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xbf,
  0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0x7f, 0xff, 0xff, 0xff, 0xff,
  0xfc, 0xfc, 0xfc, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xff,
  0xfd, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xfb, 0xff, 0xff, 0xff,
  0xfc, 0xfc, 0xfc, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xff,
  0xef, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xdf, 0xff, 0xff, 0xff,
  0xfc, 0xfc, 0xfc, 0xff, 0xbf, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xff,
  0x7f, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xfe, 0xff, 0xff,
  0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xff,
  0xff, 0xfb, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xf7, 0xff, 0xff,
  0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xef, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xff,
  0xff, 0xdf, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xbf, 0xff, 0xff,
  0xfc, 0xfc, 0xfc, 0xff, 0xff, 0x7f, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xff,
  0xff, 0xff, 0xfe, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xfd, 0xff,
  0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xfc, 0xfc, 0xfc, 0xff,
  0xff, 0xff, 0xf7, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xef, 0xff,
  0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xdf, 0xff, 0xfc, 0xfc, 0xfc, 0xff,
  0xff, 0xff, 0xbf, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0x7f, 0xff,
  0x00, 0x00, 0x00, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x10, 0x18, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x38, 0x30, 0x38, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf8, 0xe8, 0xf8, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xf8, 0xd8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xf8, 0xb8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0x78, 0xf8, 0xff,
  0xff, 0xff, 0xff, 0xff, 0x08, 0x08, 0x00, 0x07, 0x07, 0x00, 0x00, 0x00,
  0x38, 0x38, 0x38, 0x1f, 0x1f, 0x00, 0x00, 0x00, 0x38, 0x38, 0x38, 0x1c,
  0x00, 0x00, 0x00, 0x00, 0x78, 0x78, 0x70, 0x7e, 0x00, 0x00, 0x00, 0x00,
  0xf8, 0xf8, 0xe8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xd8, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xb8, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xf8, 0xf8, 0x78, 0xff, 0xff, 0xff, 0xff, 0xff, 0x38, 0x38, 0x38, 0x3e,
  0x0f, 0x0f, 0x00, 0x00, 0x78, 0x78, 0x78, 0x7d, 0x3f, 0x1f, 0x00, 0x00,
  0xf8, 0xf8, 0xf8, 0xfb, 0x7e, 0x08, 0x00, 0x00, 0xf8, 0xf8, 0xf8, 0xf7,
  0xff, 0xfc, 0x00, 0x00, 0xf8, 0xf8, 0xf8, 0xef, 0xff, 0xff, 0xff, 0xff,
  0xf8, 0xf8, 0xf8, 0xdf, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xbf,
  0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0x7f, 0xff, 0xff, 0xff, 0xff,
  0xf8, 0xf8, 0xf8, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xff,
  0xfd, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xfb, 0xff, 0xff, 0xff,
  0xf8, 0xf8, 0xf8, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xff,
  0xef, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xdf, 0xff, 0xff, 0xff,
  0xf8, 0xf8, 0xf8, 0xff, 0xbf, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xff,
  0x7f, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xfe, 0xff, 0xff,
  0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xff,
  0xff, 0xfb, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xf7, 0xff, 0xff,
  0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xef, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xff,
  0xff, 0xdf, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xbf, 0xff, 0xff,
  0xf8, 0xf8, 0xf8, 0xff, 0xff, 0x7f, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xff,
  0xff, 0xff, 0xfe, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xfd, 0xff,
  0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xf8, 0xf8, 0xf8, 0xff,
  0xff, 0xff, 0xf7, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xef, 0xff,
  0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xdf, 0xff, 0xf8, 0xf8, 0xf8, 0xff,
  0xff, 0xff, 0xbf, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0x7f, 0xff,
  0x01, 0x00, 0x01, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x20, 0x30, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x70, 0x60, 0x70, 0x00,
  0x00, 0x00, 0x00, 0x00, 0xf1, 0xd1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xf1, 0xb1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf1, 0x71, 0xf1, 0xff,
  0xff, 0xff, 0xff, 0xff, 0x01, 0x01, 0x00, 0x0f, 0x0f, 0x00, 0x00, 0x00,
  0x11, 0x11, 0x01, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x71, 0x71, 0x71, 0x3f,
  0x1e, 0x00, 0x00, 0x00, 0x71, 0x71, 0x70, 0x38, 0x00, 0x00, 0x00, 0x00,
  0xf0, 0xf0, 0xe0, 0xfc, 0x00, 0x00, 0x00, 0x00, 0xf1, 0xf1, 0xd1, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xb1, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xf1, 0xf1, 0x71, 0xff, 0xff, 0xff, 0xff, 0xff, 0x31, 0x31, 0x31, 0x3e,
  0x1f, 0x1f, 0x00, 0x00, 0x71, 0x71, 0x71, 0x7d, 0x1f, 0x01, 0x00, 0x00,
  0xf1, 0xf1, 0xf1, 0xfb, 0x7f, 0x3e, 0x00, 0x00, 0xf1, 0xf1, 0xf1, 0xf7,
  0xfc, 0x10, 0x00, 0x00, 0xf1, 0xf1, 0xf1, 0xef, 0xfe, 0xf8, 0x00, 0x00,
  0xf1, 0xf1, 0xf1, 0xdf, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xbf,
  0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0x7f, 0xff, 0xff, 0xff, 0xff,
  0xf1, 0xf1, 0xf1, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xff,
  0xfd, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xfb, 0xff, 0xff, 0xff,
  0xf1, 0xf1, 0xf1, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xff,
  0xef, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xdf, 0xff, 0xff, 0xff,
  0xf1, 0xf1, 0xf1, 0xff, 0xbf, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xff,
  0x7f, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xfe, 0xff, 0xff,
  0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xff,
  0xff, 0xfb, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xf7, 0xff, 0xff,
  0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xef, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xff,
  0xff, 0xdf, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xbf, 0xff, 0xff,
  0xf1, 0xf1, 0xf1, 0xff, 0xff, 0x7f, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xff,
  0xff, 0xff, 0xfe, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xfd, 0xff,
  0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xf1, 0xf1, 0xf1, 0xff,
  0xff, 0xff, 0xf7, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xef, 0xff,
  0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xdf, 0xff, 0xf1, 0xf1, 0xf1, 0xff,
  0xff, 0xff, 0xbf, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0x7f, 0xff,
  0xe3, 0xe2, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xff, 0x03, 0x01, 0x03, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x01, 0x03, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x60, 0x60, 0x00,
  0x00, 0x00, 0x00, 0x00, 0xe0, 0xc0, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x00,
  0xe3, 0xa3, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe3, 0x63, 0xe3, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe2, 0xff, 0xff, 0xff, 0xff, 0xff,
  0x03, 0x03, 0x01, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x23, 0x23, 0x03, 0x0f,
  0x00, 0x00, 0x00, 0x00, 0xe3, 0xe3, 0xe3, 0x7f, 0x3c, 0x00, 0x00, 0x00,
  0xe2, 0xe2, 0xe0, 0x78, 0x00, 0x00, 0x00, 0x00, 0xe0, 0xe0, 0xc0, 0xf8,
  0x00, 0x00, 0x00, 0x00, 0xe3, 0xe3, 0xa3, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xe3, 0xe3, 0x63, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xfe,
  0xff, 0xff, 0xff, 0xff, 0x63, 0x63, 0x63, 0x7d, 0x1f, 0x07, 0x00, 0x00,
  0xe3, 0xe3, 0xe3, 0xfb, 0x3f, 0x02, 0x00, 0x00, 0xe3, 0xe3, 0xe3, 0xf7,
  0xff, 0x7c, 0x00, 0x00, 0xe3, 0xe3, 0xe3, 0xef, 0xfe, 0x20, 0x00, 0x00,
  0xe3, 0xe3, 0xe3, 0xdf, 0xfc, 0xf0, 0x00, 0x00, 0xe3, 0xe3, 0xe3, 0xbf,
  0xff, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0x7f, 0xff, 0xff, 0xff, 0xff,
  0xe3, 0xe3, 0xe3, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xff,
  0xfd, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xfb, 0xff, 0xff, 0xff,
  0xe3, 0xe3, 0xe3, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xff,
  0xef, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xdf, 0xff, 0xff, 0xff,
  0xe3, 0xe3, 0xe3, 0xff, 0xbf, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xff,
  0x7f, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xfe, 0xff, 0xff,
  0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xff,
  0xff, 0xfb, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xf7, 0xff, 0xff,
  0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xef, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xff,
  0xff, 0xdf, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xbf, 0xff, 0xff,
  0xe3, 0xe3, 0xe3, 0xff, 0xff, 0x7f, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xff,
  0xff, 0xff, 0xfe, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xfd, 0xff,
  0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xe3, 0xe3, 0xe3, 0xff,
  0xff, 0xff, 0xf7, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xef, 0xff,
  0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xdf, 0xff, 0xe3, 0xe3, 0xe3, 0xff,
  0xff, 0xff, 0xbf, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0x7f, 0xff,
  0x0f, 0x0c, 0x0c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x0c, 0x0c, 0x0c,
  0x00, 0x00, 0x00, 0x00, 0x1f, 0x18, 0x1c, 0x00, 0x00, 0x00, 0x00, 0x00,
  0xff, 0xf4, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xec, 0xfc, 0xfc,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xdc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xbc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0x7c, 0xfc, 0xfc,
  0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x1f, 0x1c, 0x1c, 0x1c, 0x1e, 0x00, 0x00, 0x00, 0x3f, 0x3c, 0x38, 0x3c,
  0x00, 0x00, 0x00, 0x00, 0xff, 0xfc, 0xf4, 0xfc, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xfc, 0xec, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xdc, 0xfc,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xbc, 0xfc, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xfc, 0x7c, 0xfc, 0xff, 0xff, 0xff, 0xff, 0x3f, 0x3c, 0x3c, 0x3c,
  0x3f, 0x3f, 0x3f, 0x00, 0x7f, 0x7c, 0x7c, 0x7c, 0x3f, 0x3f, 0x00, 0x00,
  0xff, 0xfc, 0xfc, 0xf8, 0xff, 0xfe, 0x00, 0x00, 0xff, 0xfc, 0xfc, 0xf4,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xec, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xfc, 0xfc, 0xdc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xbc,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0x7c, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xfc, 0xfc, 0xfc, 0xfe, 0xff, 0x7f, 0x7f, 0xff, 0xfc, 0xfc, 0xfc,
  0xfd, 0xff, 0x7f, 0x00, 0xff, 0xfc, 0xfc, 0xfc, 0xfb, 0xff, 0xff, 0xff,
  0xff, 0xfc, 0xfc, 0xfc, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc,
  0xef, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xdf, 0xff, 0xff, 0xff,
  0xff, 0xfc, 0xfc, 0xfc, 0xbf, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc,
  0x7f, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xfe, 0xff, 0xff,
  0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc,
  0xff, 0xfb, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xf7, 0xff, 0xff,
  0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xef, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc,
  0xff, 0xdf, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xbf, 0xff, 0xff,
  0xff, 0xfc, 0xfc, 0xfc, 0xff, 0x7f, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc,
  0xff, 0xff, 0xfe, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xfd, 0xff,
  0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xfc, 0xfc, 0xfc,
  0xff, 0xff, 0xf7, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xef, 0xff,
  0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xdf, 0xff, 0xff, 0xfc, 0xfc, 0xfc,
  0xff, 0xff, 0xbf, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0x7f, 0xff,
  0x07, 0x08, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x08, 0x08, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x1f, 0x18, 0x18, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x3e, 0x30, 0x38, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xe8, 0xf8, 0xf8,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xd8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xb8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0x78, 0xf8, 0xf8,
  0xff, 0xff, 0xff, 0xff, 0x1f, 0x18, 0x18, 0x00, 0x07, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x3f, 0x38, 0x38, 0x38,
  0x00, 0x00, 0x00, 0x00, 0x7f, 0x78, 0x70, 0x78, 0x00, 0x00, 0x00, 0x00,
  0xff, 0xf8, 0xe8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xd8, 0xf8,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xb8, 0xf8, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xf8, 0x78, 0xf8, 0xff, 0xff, 0xff, 0xff, 0x3f, 0x38, 0x38, 0x38,
  0x0f, 0x0f, 0x00, 0x00, 0x7f, 0x78, 0x78, 0x78, 0x7f, 0x1f, 0x03, 0x00,
  0xff, 0xf8, 0xf8, 0xf8, 0x7c, 0x08, 0x00, 0x00, 0xff, 0xf8, 0xf8, 0xf0,
  0xfe, 0xfc, 0x00, 0x00, 0xff, 0xf8, 0xf8, 0xe8, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xf8, 0xf8, 0xd8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xb8,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0x78, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xf8, 0xf8, 0xf8, 0xfe, 0xff, 0x1f, 0x00, 0xff, 0xf8, 0xf8, 0xf8,
  0xfd, 0xff, 0x3f, 0x07, 0xff, 0xf8, 0xf8, 0xf8, 0xfb, 0xff, 0x1d, 0x00,
  0xff, 0xf8, 0xf8, 0xf8, 0xf7, 0xff, 0xfe, 0xfe, 0xff, 0xf8, 0xf8, 0xf8,
  0xef, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xdf, 0xff, 0xff, 0xff,
  0xff, 0xf8, 0xf8, 0xf8, 0xbf, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8,
  0x7f, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xfe, 0xff, 0xff,
  0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8,
  0xff, 0xfb, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xf7, 0xff, 0xff,
  0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xef, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8,
  0xff, 0xdf, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xbf, 0xff, 0xff,
  0xff, 0xf8, 0xf8, 0xf8, 0xff, 0x7f, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8,
  0xff, 0xff, 0xfe, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xfd, 0xff,
  0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xf8, 0xf8, 0xf8,
  0xff, 0xff, 0xf7, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xef, 0xff,
  0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xdf, 0xff, 0xff, 0xf8, 0xf8, 0xf8,
  0xff, 0xff, 0xbf, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0x7f, 0xff,
  0x07, 0x00, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x01, 0x01, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x1f, 0x11, 0x11, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x3e, 0x30, 0x30, 0x00, 0x00, 0x00, 0x00, 0x00, 0x7c, 0x60, 0x70, 0x00,
  0x00, 0x00, 0x00, 0x00, 0xff, 0xd1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xb1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xff, 0x71, 0xf1, 0xf1,
  0xff, 0xff, 0xff, 0xff, 0x0f, 0x01, 0x00, 0x01, 0x07, 0x00, 0x00, 0x00,
  0x1f, 0x11, 0x11, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x7f, 0x71, 0x71, 0x70, 0x00, 0x00, 0x00, 0x00,
  0xfe, 0xf0, 0xe0, 0xf0, 0x00, 0x00, 0x00, 0x00, 0xff, 0xf1, 0xd1, 0xf1,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xb1, 0xf1, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xf1, 0x71, 0xf1, 0xff, 0xff, 0xff, 0xff, 0x3f, 0x31, 0x31, 0x30,
  0x0f, 0x0f, 0x00, 0x00, 0x7f, 0x71, 0x71, 0x71, 0x0f, 0x01, 0x00, 0x00,
  0xff, 0xf1, 0xf1, 0xf1, 0xff, 0x3e, 0x00, 0x00, 0xff, 0xf1, 0xf1, 0xf1,
  0xfc, 0x10, 0x00, 0x00, 0xff, 0xf1, 0xf1, 0xe1, 0xfc, 0xf8, 0x00, 0x00,
  0xff, 0xf1, 0xf1, 0xd1, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xb1,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0x71, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xf1, 0xf1, 0xf1, 0xfe, 0xff, 0x1f, 0x1f, 0xff, 0xf1, 0xf1, 0xf1,
  0xfd, 0xff, 0x0f, 0x01, 0xff, 0xf1, 0xf1, 0xf1, 0xfb, 0xff, 0x7f, 0x00,
  0xff, 0xf1, 0xf1, 0xf1, 0xf7, 0xff, 0x3a, 0x00, 0xff, 0xf1, 0xf1, 0xf1,
  0xef, 0xff, 0xfc, 0xfc, 0xff, 0xf1, 0xf1, 0xf1, 0xdf, 0xff, 0xff, 0xff,
  0xff, 0xf1, 0xf1, 0xf1, 0xbf, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf1,
  0x7f, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xfe, 0xff, 0xff,
  0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf1,
  0xff, 0xfb, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xf7, 0xff, 0xff,
  0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xef, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf1,
  0xff, 0xdf, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xbf, 0xff, 0xff,
  0xff, 0xf1, 0xf1, 0xf1, 0xff, 0x7f, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf1,
  0xff, 0xff, 0xfe, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xfd, 0xff,
  0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xf1, 0xf1, 0xf1,
  0xff, 0xff, 0xf7, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xef, 0xff,
  0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xdf, 0xff, 0xff, 0xf1, 0xf1, 0xf1,
  0xff, 0xff, 0xbf, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0x7f, 0xff,
  0xff, 0xe2, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff, 0x0f, 0x01, 0x03, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x1f, 0x03, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x3e, 0x22, 0x22, 0x00, 0x00, 0x00, 0x00, 0x00, 0x7c, 0x60, 0x60, 0x00,
  0x00, 0x00, 0x00, 0x00, 0xf8, 0xc0, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x00,
  0xff, 0xa3, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xff, 0x63, 0xe3, 0xe3,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xe2, 0xe3, 0xff, 0xff, 0xff, 0xff,
  0x1f, 0x03, 0x01, 0x03, 0x00, 0x00, 0x00, 0x00, 0x3f, 0x23, 0x23, 0x03,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0xfe, 0xe2, 0xe2, 0xe0, 0x00, 0x00, 0x00, 0x00, 0xfc, 0xe0, 0xc0, 0xe0,
  0x00, 0x00, 0x00, 0x00, 0xff, 0xe3, 0xa3, 0xe3, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xe3, 0x63, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe2,
  0xff, 0xff, 0xff, 0xff, 0x7f, 0x63, 0x63, 0x61, 0x0f, 0x07, 0x00, 0x00,
  0xff, 0xe3, 0xe3, 0xe3, 0x0f, 0x02, 0x00, 0x00, 0xff, 0xe3, 0xe3, 0xe3,
  0xff, 0x3c, 0x00, 0x00, 0xff, 0xe3, 0xe3, 0xe3, 0xf8, 0x20, 0x00, 0x00,
  0xff, 0xe3, 0xe3, 0xc3, 0xf8, 0xf0, 0x00, 0x00, 0xff, 0xe3, 0xe3, 0xa3,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0x63, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xe3, 0xe3, 0xe3, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe3,
  0xfd, 0xff, 0x1f, 0x0f, 0xff, 0xe3, 0xe3, 0xe3, 0xfb, 0xff, 0x17, 0x00,
  0xff, 0xe3, 0xe3, 0xe3, 0xf7, 0xff, 0x7e, 0x00, 0xff, 0xe3, 0xe3, 0xe3,
  0xef, 0xff, 0x74, 0x00, 0xff, 0xe3, 0xe3, 0xe3, 0xdf, 0xff, 0xfc, 0xf8,
  0xff, 0xe3, 0xe3, 0xe3, 0xbf, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe3,
  0x7f, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xfe, 0xff, 0xff,
  0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe3,
  0xff, 0xfb, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xf7, 0xff, 0xff,
  0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xef, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe3,
  0xff, 0xdf, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xbf, 0xff, 0xff,
  0xff, 0xe3, 0xe3, 0xe3, 0xff, 0x7f, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe3,
  0xff, 0xff, 0xfe, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xfd, 0xff,
  0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xe3, 0xe3, 0xe3,
  0xff, 0xff, 0xf7, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xef, 0xff,
  0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xdf, 0xff, 0xff, 0xe3, 0xe3, 0xe3,
  0xff, 0xff, 0xbf, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0x7f, 0xff,
  0x1f, 0x0e, 0x1c, 0x1c, 0x00, 0x00, 0x00, 0x00, 0x1f, 0x1d, 0x1c, 0x1c,
  0x0c, 0x00, 0x00, 0x00, 0x3f, 0x3b, 0x3c, 0x3c, 0x00, 0x00, 0x00, 0x00,
  0xff, 0xf7, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xef, 0xfc, 0xfc,
  0xfc, 0xff, 0xff, 0xff, 0xff, 0xdf, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff,
  0xff, 0xbf, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0x7f, 0xfc, 0xfc,
  0xfc, 0xff, 0xff, 0xff, 0x1f, 0x1f, 0x1c, 0x1c, 0x0c, 0x00, 0x00, 0x00,
  0x3f, 0x3f, 0x3c, 0x3c, 0x3c, 0x1e, 0x00, 0x00, 0x7f, 0x7f, 0x78, 0x7c,
  0x7c, 0x00, 0x00, 0x00, 0xff, 0xff, 0xf4, 0xfc, 0xfc, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xec, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xdc, 0xfc,
  0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xbc, 0xfc, 0xfc, 0xff, 0xff, 0xff,
  0xff, 0xff, 0x7c, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x7f, 0x7f, 0x7c, 0x7c, 0x7c, 0x7f, 0x1f, 0x00,
  0xff, 0xff, 0xfc, 0xf8, 0xfc, 0xfe, 0x00, 0x00, 0xff, 0xff, 0xfc, 0xf4,
  0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xec, 0xfc, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xfc, 0xdc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xbc,
  0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0x7c, 0xfc, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc,
  0xfc, 0xff, 0xff, 0x3f, 0xff, 0xff, 0xfc, 0xfc, 0xf8, 0xff, 0xff, 0x7f,
  0xff, 0xff, 0xfc, 0xfc, 0xf4, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc,
  0xec, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xdc, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xfc, 0xfc, 0xbc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc,
  0x7c, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xfe, 0xff, 0xff,
  0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc,
  0xfc, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xf7, 0xff, 0xff,
  0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xef, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc,
  0xfc, 0xdf, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xbf, 0xff, 0xff,
  0xff, 0xff, 0xfc, 0xfc, 0xfc, 0x7f, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc,
  0xfc, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xfd, 0xff,
  0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xfb, 0xff, 0xff, 0xff, 0xfc, 0xfc,
  0xfc, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xef, 0xff,
  0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xdf, 0xff, 0xff, 0xff, 0xfc, 0xfc,
  0xfc, 0xff, 0xbf, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0x7f, 0xff,
  0x0f, 0x0e, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1f, 0x1d, 0x08, 0x08,
  0x00, 0x00, 0x00, 0x00, 0x3f, 0x3b, 0x38, 0x38, 0x00, 0x00, 0x00, 0x00,
  0x7f, 0x77, 0x78, 0x78, 0x00, 0x00, 0x00, 0x00, 0xff, 0xef, 0xf8, 0xf8,
  0xf8, 0xff, 0xff, 0xff, 0xff, 0xdf, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xff,
  0xff, 0xbf, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0x7f, 0xf8, 0xf8,
  0xf8, 0xff, 0xff, 0xff, 0x1f, 0x1f, 0x18, 0x18, 0x00, 0x00, 0x00, 0x00,
  0x3f, 0x3f, 0x38, 0x18, 0x18, 0x00, 0x00, 0x00, 0x7f, 0x7f, 0x78, 0x78,
  0x78, 0x00, 0x00, 0x00, 0xff, 0xff, 0xf0, 0xf8, 0xf8, 0x00, 0x00, 0x00,
  0xff, 0xff, 0xe8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xd8, 0xf8,
  0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xb8, 0xf8, 0xf8, 0xff, 0xff, 0xff,
  0xff, 0xff, 0x78, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0x3f, 0x3f, 0x38, 0x38,
  0x38, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xfc, 0x00, 0x00, 0xff, 0xff, 0xf8, 0xf0,
  0xf8, 0xfc, 0x00, 0x00, 0xff, 0xff, 0xf8, 0xe8, 0xf8, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xf8, 0xd8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xb8,
  0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0x78, 0xf8, 0xff, 0xff, 0xff,
  0x7f, 0x7f, 0x78, 0x78, 0x78, 0x7f, 0x0f, 0x00, 0xff, 0xff, 0xf8, 0xf8,
  0xf8, 0xff, 0x7f, 0x0f, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xfe, 0xfe, 0x00,
  0xff, 0xff, 0xf8, 0xf8, 0xf0, 0xff, 0xfe, 0xfe, 0xff, 0xff, 0xf8, 0xf8,
  0xe8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xd8, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xf8, 0xf8, 0xb8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8,
  0x78, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xfe, 0xff, 0xff,
  0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8,
  0xf8, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xf7, 0xff, 0xff,
  0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xef, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8,
  0xf8, 0xdf, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xbf, 0xff, 0xff,
  0xff, 0xff, 0xf8, 0xf8, 0xf8, 0x7f, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8,
  0xf8, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xfd, 0xff,
  0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xfb, 0xff, 0xff, 0xff, 0xf8, 0xf8,
  0xf8, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xef, 0xff,
  0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xdf, 0xff, 0xff, 0xff, 0xf8, 0xf8,
  0xf8, 0xff, 0xbf, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0x7f, 0xff,
  0x0f, 0x0e, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x1f, 0x1d, 0x01, 0x01,
  0x00, 0x00, 0x00, 0x00, 0x3f, 0x3b, 0x11, 0x11, 0x00, 0x00, 0x00, 0x00,
  0x7f, 0x77, 0x70, 0x70, 0x00, 0x00, 0x00, 0x00, 0xfe, 0xee, 0xf0, 0xf0,
  0x00, 0x00, 0x00, 0x00, 0xff, 0xdf, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xff,
  0xff, 0xbf, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0x7f, 0xf1, 0xf1,
  0xf1, 0xff, 0xff, 0xff, 0x1f, 0x1f, 0x10, 0x01, 0x01, 0x00, 0x00, 0x00,
  0x3f, 0x3f, 0x31, 0x11, 0x01, 0x00, 0x00, 0x00, 0x7f, 0x7f, 0x71, 0x31,
  0x31, 0x00, 0x00, 0x00, 0xff, 0xff, 0xf1, 0xf1, 0xf0, 0x00, 0x00, 0x00,
  0xff, 0xff, 0xe1, 0xf0, 0xf0, 0x00, 0x00, 0x00, 0xff, 0xff, 0xd1, 0xf1,
  0xf1, 0xff, 0xff, 0xff, 0xff, 0xff, 0xb1, 0xf1, 0xf1, 0xff, 0xff, 0xff,
  0xff, 0xff, 0x71, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0x3f, 0x3f, 0x31, 0x30,
  0x01, 0x07, 0x00, 0x00, 0x7f, 0x7f, 0x71, 0x71, 0x31, 0x03, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xf1, 0xf1,
  0xf1, 0xf8, 0x00, 0x00, 0xff, 0xff, 0xf1, 0xe1, 0xf0, 0xf8, 0x00, 0x00,
  0xff, 0xff, 0xf1, 0xd1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xb1,
  0xf1, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf1, 0x71, 0xf1, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xf1, 0xf1, 0xf0, 0xff, 0x0f, 0x0f, 0xff, 0xff, 0xf1, 0xf1,
  0xf1, 0xff, 0x07, 0x01, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0x1f,
  0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xfc, 0x00, 0xff, 0xff, 0xf1, 0xf1,
  0xe1, 0xff, 0xfc, 0xfc, 0xff, 0xff, 0xf1, 0xf1, 0xd1, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xf1, 0xf1, 0xb1, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1,
  0x71, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xfe, 0xff, 0xff,
  0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1,
  0xf1, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xf7, 0xff, 0xff,
  0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xef, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1,
  0xf1, 0xdf, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xbf, 0xff, 0xff,
  0xff, 0xff, 0xf1, 0xf1, 0xf1, 0x7f, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1,
  0xf1, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xfd, 0xff,
  0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xfb, 0xff, 0xff, 0xff, 0xf1, 0xf1,
  0xf1, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xef, 0xff,
  0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xdf, 0xff, 0xff, 0xff, 0xf1, 0xf1,
  0xf1, 0xff, 0xbf, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0x7f, 0xff,
  0xff, 0xfe, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0x1f, 0x1d, 0x03, 0x03,
  0x00, 0x00, 0x00, 0x00, 0x3f, 0x3b, 0x03, 0x03, 0x00, 0x00, 0x00, 0x00,
  0x7f, 0x77, 0x22, 0x22, 0x00, 0x00, 0x00, 0x00, 0xfe, 0xee, 0xe0, 0xe0,
  0x00, 0x00, 0x00, 0x00, 0xfc, 0xdc, 0xe0, 0xe0, 0x00, 0x00, 0x00, 0x00,
  0xff, 0xbf, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff, 0x7f, 0xe3, 0xe3,
  0xe3, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe2, 0xe3, 0xe3, 0xff, 0xff, 0xff,
  0x3f, 0x3f, 0x21, 0x03, 0x03, 0x00, 0x00, 0x00, 0x7f, 0x7f, 0x63, 0x23,
  0x03, 0x00, 0x00, 0x00, 0xff, 0xff, 0xe3, 0x63, 0x63, 0x00, 0x00, 0x00,
  0xff, 0xff, 0xe3, 0xe2, 0xe0, 0x00, 0x00, 0x00, 0xfe, 0xfe, 0xc2, 0xe0,
  0xe0, 0x00, 0x00, 0x00, 0xff, 0xff, 0xa3, 0xe3, 0xe3, 0xff, 0xff, 0xff,
  0xff, 0xff, 0x63, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xe2,
  0xe3, 0xff, 0xff, 0xff, 0x7f, 0x7f, 0x63, 0x61, 0x03, 0x07, 0x00, 0x00,
  0xff, 0xff, 0xe3, 0xe3, 0x63, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xf0, 0x00, 0x00,
  0xff, 0xff, 0xe3, 0xc3, 0xe0, 0xf0, 0x00, 0x00, 0xff, 0xff, 0xe3, 0xa3,
  0xe3, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe3, 0x63, 0xe3, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xe3, 0xe3, 0xe2, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xe3,
  0xe1, 0xff, 0x0f, 0x0f, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0x0f, 0x00,
  0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0x3e, 0xff, 0xff, 0xe3, 0xe3,
  0xe3, 0xff, 0xf8, 0x00, 0xff, 0xff, 0xe3, 0xe3, 0xc3, 0xff, 0xf8, 0xf8,
  0xff, 0xff, 0xe3, 0xe3, 0xa3, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xe3,
  0x63, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xfe, 0xff, 0xff,
  0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xfd, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xe3,
  0xe3, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xf7, 0xff, 0xff,
  0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xef, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xe3,
  0xe3, 0xdf, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xbf, 0xff, 0xff,
  0xff, 0xff, 0xe3, 0xe3, 0xe3, 0x7f, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xe3,
  0xe3, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xfd, 0xff,
  0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xfb, 0xff, 0xff, 0xff, 0xe3, 0xe3,
  0xe3, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xef, 0xff,
  0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xdf, 0xff, 0xff, 0xff, 0xe3, 0xe3,
  0xe3, 0xff, 0xbf, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0x7f, 0xff,
  0xff, 0xfe, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xfc,
  0xfc, 0xfc, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xff,
  0xff, 0xf7, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xef, 0xff, 0xfc,
  0xfc, 0xfc, 0xff, 0xff, 0xff, 0xdf, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xff,
  0xff, 0xbf, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0x7f, 0xff, 0xfc,
  0xfc, 0xfc, 0xff, 0xff, 0x3f, 0x3f, 0x3e, 0x3c, 0x3c, 0x0c, 0x00, 0x00,
  0x7f, 0x7f, 0x7d, 0x7c, 0x7c, 0x7c, 0x05, 0x00, 0xff, 0xff, 0xfb, 0xfc,
  0xfc, 0xfc, 0xfe, 0x00, 0xff, 0xff, 0xf7, 0xfc, 0xfc, 0xfc, 0xff, 0xff,
  0xff, 0xff, 0xef, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xdf, 0xfc,
  0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xbf, 0xfc, 0xfc, 0xfc, 0xff, 0xff,
  0xff, 0xff, 0x7f, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0x7f, 0x7f, 0x7f, 0x7c,
  0x7c, 0x7c, 0x0e, 0x00, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0x7f, 0x00,
  0xff, 0xff, 0xff, 0xf8, 0xfc, 0xfc, 0xfe, 0x00, 0xff, 0xff, 0xff, 0xf4,
  0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xec, 0xfc, 0xfc, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xdc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xbc,
  0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0x7c, 0xfc, 0xfc, 0xff, 0xff,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xfc,
  0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xf8, 0xfc, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xfc, 0xf4, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc,
  0xec, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xdc, 0xfc, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xfc, 0xbc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc,
  0x7c, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc,
  0xfc, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xf4, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xfc, 0xfc, 0xec, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc,
  0xfc, 0xdc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xbc, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xfc, 0xfc, 0x7c, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc,
  0xfc, 0xfc, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xfd, 0xff,
  0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xfc,
  0xfc, 0xfc, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xef, 0xff,
  0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xdf, 0xff, 0xff, 0xff, 0xff, 0xfc,
  0xfc, 0xfc, 0xbf, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0x7f, 0xff,
  0xff, 0xfe, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xf8,
  0xf8, 0xf8, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xff,
  0xff, 0xf7, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xef, 0xff, 0xf8,
  0xf8, 0xf8, 0xff, 0xff, 0xff, 0xdf, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xff,
  0xff, 0xbf, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0x7f, 0xff, 0xf8,
  0xf8, 0xf8, 0xff, 0xff, 0x3f, 0x3f, 0x3e, 0x38, 0x18, 0x00, 0x00, 0x00,
  0x7f, 0x7f, 0x7d, 0x78, 0x38, 0x18, 0x00, 0x00, 0xff, 0xff, 0xfb, 0xf8,
  0xf8, 0xf8, 0x00, 0x00, 0xff, 0xff, 0xf7, 0xf8, 0xf8, 0xf8, 0xfc, 0x00,
  0xff, 0xff, 0xef, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xdf, 0xf8,
  0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xbf, 0xf8, 0xf8, 0xf8, 0xff, 0xff,
  0xff, 0xff, 0x7f, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0x7f, 0x7f, 0x7f, 0x78,
  0x38, 0x18, 0x00, 0x00, 0x7f, 0x7f, 0x7f, 0x78, 0x78, 0x38, 0x00, 0x00,
  0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xfc, 0x00, 0xff, 0xff, 0xff, 0xf0,
  0xf8, 0xf8, 0xfc, 0x00, 0xff, 0xff, 0xff, 0xe8, 0xf8, 0xf8, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xd8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xb8,
  0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0x78, 0xf8, 0xf8, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xf8, 0x78, 0x78, 0x3f, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xf8, 0xf0, 0xf8, 0xfe, 0xfe, 0xff, 0xff, 0xff, 0xf8,
  0xe8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xd8, 0xf8, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xf8, 0xb8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8,
  0x78, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0x7f,
  0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8,
  0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf0, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xf8, 0xf8, 0xe8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8,
  0xf8, 0xd8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xb8, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xf8, 0xf8, 0x78, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8,
  0xf8, 0xf8, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xfd, 0xff,
  0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xf8,
  0xf8, 0xf8, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xef, 0xff,
  0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xdf, 0xff, 0xff, 0xff, 0xff, 0xf8,
  0xf8, 0xf8, 0xbf, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0x7f, 0xff,
  0xff, 0xfe, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xf1,
  0xf1, 0xf1, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xff,
  0xff, 0xf7, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xef, 0xff, 0xf1,
  0xf1, 0xf1, 0xff, 0xff, 0xff, 0xdf, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xff,
  0xff, 0xbf, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0x7f, 0xff, 0xf1,
  0xf1, 0xf1, 0xff, 0xff, 0x3f, 0x3f, 0x3e, 0x31, 0x01, 0x01, 0x00, 0x00,
  0x7f, 0x7f, 0x7d, 0x71, 0x31, 0x01, 0x00, 0x00, 0xff, 0xff, 0xfb, 0xf1,
  0x71, 0x31, 0x00, 0x00, 0xff, 0xff, 0xf7, 0xf1, 0xf1, 0xf0, 0x00, 0x00,
  0xff, 0xff, 0xef, 0xf1, 0xf0, 0xf0, 0xf8, 0x00, 0xff, 0xff, 0xdf, 0xf1,
  0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xbf, 0xf1, 0xf1, 0xf1, 0xff, 0xff,
  0xff, 0xff, 0x7f, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0x7f, 0x7f, 0x7f, 0x70,
  0x71, 0x01, 0x03, 0x00, 0xff, 0xff, 0xff, 0x71, 0x71, 0x01, 0x03, 0x00,
  0xff, 0xff, 0xff, 0xf1, 0xf1, 0x71, 0x00, 0x00, 0xff, 0xff, 0xff, 0xf1,
  0xf1, 0xf0, 0xf8, 0x00, 0xff, 0xff, 0xff, 0xe1, 0xf1, 0xf0, 0xf8, 0x00,
  0xff, 0xff, 0xff, 0xd1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xff, 0xb1,
  0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xff, 0x71, 0xf1, 0xf1, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xf1, 0xf0, 0xf1, 0x0f, 0x07, 0xff, 0xff, 0xff, 0xf1,
  0xf1, 0xf1, 0x0f, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xfe, 0xfe, 0xff, 0xff, 0xff, 0xf1,
  0xe1, 0xf1, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xf1, 0xd1, 0xf1, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xf1, 0xb1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf1,
  0x71, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf0, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0x1f, 0xff, 0xff, 0xff, 0xf1,
  0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xf1, 0xf1, 0xe1, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf1,
  0xf1, 0xd1, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xb1, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xf1, 0xf1, 0x71, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf1,
  0xf1, 0xf1, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xfd, 0xff,
  0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xf1,
  0xf1, 0xf1, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xef, 0xff,
  0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xdf, 0xff, 0xff, 0xff, 0xff, 0xf1,
  0xf1, 0xf1, 0xbf, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0x7f, 0xff,
  0xff, 0xfe, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xe3,
  0xe3, 0xe3, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xff,
  0xff, 0xf7, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xef, 0xff, 0xe3,
  0xe3, 0xe3, 0xff, 0xff, 0xff, 0xdf, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xff,
  0xff, 0xbf, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0x7f, 0xff, 0xe3,
  0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xe3, 0xe3, 0xe3, 0xff, 0xff,
  0x7f, 0x7f, 0x7d, 0x63, 0x03, 0x03, 0x07, 0x00, 0xff, 0xff, 0xfb, 0xe3,
  0x63, 0x03, 0x00, 0x00, 0xff, 0xff, 0xf7, 0xe3, 0xe3, 0x63, 0x00, 0x00,
  0xff, 0xff, 0xef, 0xe3, 0xe3, 0xe0, 0x00, 0x00, 0xff, 0xff, 0xdf, 0xe3,
  0xe0, 0xe0, 0xf0, 0x00, 0xff, 0xff, 0xbf, 0xe3, 0xe3, 0xe3, 0xff, 0xff,
  0xff, 0xff, 0x7f, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe2,
  0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe1, 0xe3, 0x03, 0x07, 0x00,
  0xff, 0xff, 0xff, 0xe3, 0xe3, 0x03, 0x07, 0x00, 0xff, 0xff, 0xff, 0xe3,
  0xe3, 0xe3, 0x00, 0x00, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe0, 0xf0, 0x00,
  0xff, 0xff, 0xff, 0xc3, 0xe3, 0xe0, 0xf0, 0x00, 0xff, 0xff, 0xff, 0xa3,
  0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xff, 0x63, 0xe3, 0xe3, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xe3, 0xe2, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe3,
  0xe1, 0xe3, 0x0f, 0x0f, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0x1f, 0x1f,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xe3,
  0xe3, 0xe3, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xe3, 0xc3, 0xe3, 0xf8, 0xf8,
  0xff, 0xff, 0xff, 0xe3, 0xa3, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe3,
  0x63, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe2, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe1, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe3,
  0xe3, 0xe3, 0xff, 0x3f, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xe3,
  0xe3, 0xc3, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xa3, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xe3, 0xe3, 0x63, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe3,
  0xe3, 0xe3, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xfd, 0xff,
  0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xe3,
  0xe3, 0xe3, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xef, 0xff,
  0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xdf, 0xff, 0xff, 0xff, 0xff, 0xe3,
  0xe3, 0xe3, 0xbf, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0x7f, 0xff,
  0xff, 0xfe, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xfd, 0xff, 0xff,
  0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xff,
  0xff, 0xf7, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xef, 0xff, 0xff,
  0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xdf, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xff,
  0xff, 0xbf, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0x7f, 0xff, 0xff,
  0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xfe, 0xff, 0xfc, 0xfc, 0xfc, 0xff,
  0xff, 0xff, 0xfd, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xfb, 0xff,
  0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xfc, 0xfc, 0xfc, 0xff,
  0xff, 0xff, 0xef, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xdf, 0xff,
  0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xbf, 0xff, 0xfc, 0xfc, 0xfc, 0xff,
  0xff, 0xff, 0x7f, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xfe,
  0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xfc, 0xfc, 0xfc, 0xff,
  0xff, 0xff, 0xff, 0xfb, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xf7,
  0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xef, 0xfc, 0xfc, 0xfc, 0xff,
  0xff, 0xff, 0xff, 0xdf, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xbf,
  0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0x7f, 0xfc, 0xfc, 0xfc, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xfc, 0xfc, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xf4, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xec, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xdc, 0xfc, 0xfc, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xbc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff,
  0x7c, 0xfc, 0xfc, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfc, 0xf8, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xf4, 0xfc, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xfc, 0xec, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfc, 0xdc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xbc, 0xfc, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xfc, 0x7c, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfc, 0xfc, 0xf4, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xec, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xdc, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfc, 0xfc, 0xbc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0x7c, 0xff,
  0xff, 0xfe, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xfd, 0xff, 0xff,
  0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xff,
  0xff, 0xf7, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xef, 0xff, 0xff,
  0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xdf, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xff,
  0xff, 0xbf, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0x7f, 0xff, 0xff,
  0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xfe, 0xff, 0xf8, 0xf8, 0xf8, 0xff,
  0xff, 0xff, 0xfd, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xfb, 0xff,
  0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xf8, 0xf8, 0xf8, 0xff,
  0xff, 0xff, 0xef, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xdf, 0xff,
  0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xbf, 0xff, 0xf8, 0xf8, 0xf8, 0xff,
  0xff, 0xff, 0x7f, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xfe,
  0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xf8, 0xf8, 0xf8, 0xff,
  0xff, 0xff, 0xff, 0xfb, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xf7,
  0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xef, 0xf8, 0xf8, 0xf8, 0xff,
  0xff, 0xff, 0xff, 0xdf, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xbf,
  0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0x7f, 0xf8, 0xf8, 0xf8, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xf0, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xe8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xd8, 0xf8, 0xf8, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xb8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff,
  0x78, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xff,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff,
  0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xf0, 0xf8, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xf8, 0xe8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xf8, 0xd8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xb8, 0xf8, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xf8, 0x78, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xf8, 0xf8, 0xf0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xe8, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xd8, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xf8, 0xf8, 0xb8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0x78, 0xff,
  0xff, 0xfe, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xfd, 0xff, 0xff,
  0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xff,
  0xff, 0xf7, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xef, 0xff, 0xff,
  0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xdf, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xff,
  0xff, 0xbf, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0x7f, 0xff, 0xff,
  0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xfe, 0xff, 0xf1, 0xf1, 0xf1, 0xff,
  0xff, 0xff, 0xfd, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xfb, 0xff,
  0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xf1, 0xf1, 0xf1, 0xff,
  0xff, 0xff, 0xef, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xdf, 0xff,
  0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xbf, 0xff, 0xf1, 0xf1, 0xf1, 0xff,
  0xff, 0xff, 0x7f, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xfe,
  0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xf1, 0xf1, 0xf1, 0xff,
  0xff, 0xff, 0xff, 0xfb, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xf7,
  0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xef, 0xf1, 0xf1, 0xf1, 0xff,
  0xff, 0xff, 0xff, 0xdf, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xbf,
  0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0x7f, 0xf1, 0xf1, 0xf1, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xf0, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xe1, 0xf1, 0xf1, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xd1, 0xf1, 0xf1, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xb1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xff,
  0x71, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xf0, 0xf1, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xf1, 0xe1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xf1, 0xd1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xb1, 0xf1, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xf1, 0x71, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xf1, 0xf1, 0xf0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xe1, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xd1, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xf1, 0xf1, 0xb1, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0x71, 0xff,
  0xff, 0xfe, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xfd, 0xff, 0xff,
  0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xff,
  0xff, 0xf7, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xef, 0xff, 0xff,
  0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xdf, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xff,
  0xff, 0xbf, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0x7f, 0xff, 0xff,
  0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xfe, 0xff, 0xe3, 0xe3, 0xe3, 0xff,
  0xff, 0xff, 0xfd, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xfb, 0xff,
  0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xe3, 0xe3, 0xe3, 0xff,
  0xff, 0xff, 0xef, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xdf, 0xff,
  0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xbf, 0xff, 0xe3, 0xe3, 0xe3, 0xff,
  0xff, 0xff, 0x7f, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xfe,
  0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xe3, 0xe3, 0xe3, 0xff,
  0xff, 0xff, 0xff, 0xfb, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xf7,
  0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xef, 0xe3, 0xe3, 0xe3, 0xff,
  0xff, 0xff, 0xff, 0xdf, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xbf,
  0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff, 0x7f, 0xe3, 0xe3, 0xe3, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xe2, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xe1, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xe3, 0xe3, 0xe3, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xc3, 0xe3, 0xe3, 0xfc,
  0xff, 0xff, 0xff, 0xff, 0xa3, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xff,
  0x63, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xe2, 0xe3, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xe3, 0xe1, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xe3, 0xe3, 0xe3, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0xff, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xe3, 0xc3, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xa3, 0xe3, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xe3, 0x63, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xe3, 0xe3, 0xe2, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe1, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xc3, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xe3, 0xe3, 0xa3, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0x63, 0xff,
  0xff, 0xfe, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xfd, 0xff, 0xff,
  0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xfb, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc,
  0xff, 0xf7, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xef, 0xff, 0xff,
  0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xdf, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc,
  0xff, 0xbf, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0x7f, 0xff, 0xff,
  0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xfc, 0xfc, 0xfc,
  0xff, 0xff, 0xfd, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xfb, 0xff,
  0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xfc, 0xfc, 0xfc,
  0xff, 0xff, 0xef, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xdf, 0xff,
  0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xbf, 0xff, 0xff, 0xfc, 0xfc, 0xfc,
  0xff, 0xff, 0x7f, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xfe,
  0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xfc, 0xfc, 0xfc,
  0xff, 0xff, 0xff, 0xfb, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xf7,
  0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xef, 0xff, 0xfc, 0xfc, 0xfc,
  0xff, 0xff, 0xff, 0xdf, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xbf,
  0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0x7f, 0xff, 0xfc, 0xfc, 0xfc,
  0xff, 0xff, 0xff, 0xff, 0xfe, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff,
  0xfd, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xfc, 0xfc, 0xfc,
  0xff, 0xff, 0xff, 0xff, 0xf7, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff,
  0xef, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xdf, 0xfc, 0xfc, 0xfc,
  0xff, 0xff, 0xff, 0xff, 0xbf, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff,
  0x7f, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xf8, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf4, 0xfc, 0xfc,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xec, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xdc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xbc, 0xfc, 0xfc,
  0xff, 0xff, 0xff, 0xff, 0xff, 0x7c, 0xfc, 0xfc, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xf8, 0xfc, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xfc, 0xf4, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xec, 0xfc,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xdc, 0xfc, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xfc, 0xbc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0x7c, 0xfc,
  0xff, 0xfe, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xfd, 0xff, 0xff,
  0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xfb, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8,
  0xff, 0xf7, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xef, 0xff, 0xff,
  0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xdf, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8,
  0xff, 0xbf, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0x7f, 0xff, 0xff,
  0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xf8, 0xf8, 0xf8,
  0xff, 0xff, 0xfd, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xfb, 0xff,
  0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xf8, 0xf8, 0xf8,
  0xff, 0xff, 0xef, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xdf, 0xff,
  0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xbf, 0xff, 0xff, 0xf8, 0xf8, 0xf8,
  0xff, 0xff, 0x7f, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xfe,
  0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xf8, 0xf8, 0xf8,
  0xff, 0xff, 0xff, 0xfb, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xf7,
  0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xef, 0xff, 0xf8, 0xf8, 0xf8,
  0xff, 0xff, 0xff, 0xdf, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xbf,
  0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0x7f, 0xff, 0xf8, 0xf8, 0xf8,
  0xff, 0xff, 0xff, 0xff, 0xfe, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff,
  0xfd, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xf8, 0xf8, 0xf8,
  0xff, 0xff, 0xff, 0xff, 0xf7, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff,
  0xef, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xdf, 0xf8, 0xf8, 0xf8,
  0xff, 0xff, 0xff, 0xff, 0xbf, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff,
  0x7f, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf0, 0xf8, 0xf8,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xe8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xd8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xb8, 0xf8, 0xf8,
  0xff, 0xff, 0xff, 0xff, 0xff, 0x78, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xf8, 0xf8, 0xf8, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xf8, 0xf0, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xe8, 0xf8,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xd8, 0xf8, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xf8, 0xb8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0x78, 0xf8,
  0xff, 0xfe, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xfd, 0xff, 0xff,
  0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xfb, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf1,
  0xff, 0xf7, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xef, 0xff, 0xff,
  0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xdf, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf1,
  0xff, 0xbf, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0x7f, 0xff, 0xff,
  0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xf1, 0xf1, 0xf1,
  0xff, 0xff, 0xfd, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xfb, 0xff,
  0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xf1, 0xf1, 0xf1,
  0xff, 0xff, 0xef, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xdf, 0xff,
  0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xbf, 0xff, 0xff, 0xf1, 0xf1, 0xf1,
  0xff, 0xff, 0x7f, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xfe,
  0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xf1, 0xf1, 0xf1,
  0xff, 0xff, 0xff, 0xfb, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xf7,
  0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xef, 0xff, 0xf1, 0xf1, 0xf1,
  0xff, 0xff, 0xff, 0xdf, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xbf,
  0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0x7f, 0xff, 0xf1, 0xf1, 0xf1,
  0xff, 0xff, 0xff, 0xff, 0xfe, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff,
  0xfd, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xf1, 0xf1, 0xf1,
  0xff, 0xff, 0xff, 0xff, 0xf7, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff,
  0xef, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xdf, 0xf1, 0xf1, 0xf1,
  0xff, 0xff, 0xff, 0xff, 0xbf, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff,
  0x7f, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf0, 0xf1, 0xf1,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf1,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xe1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xd1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xff, 0xb1, 0xf1, 0xf1,
  0xff, 0xff, 0xff, 0xff, 0xff, 0x71, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xf1, 0xf0, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf1,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xe1, 0xf1,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xd1, 0xf1, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xf1, 0xb1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf1, 0x71, 0xf1,
  0xff, 0xfe, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xfd, 0xff, 0xff,
  0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xfb, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe3,
  0xff, 0xf7, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xef, 0xff, 0xff,
  0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xdf, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe3,
  0xff, 0xbf, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0x7f, 0xff, 0xff,
  0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xe3, 0xe3, 0xe3,
  0xff, 0xff, 0xfd, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xfb, 0xff,
  0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xe3, 0xe3, 0xe3,
  0xff, 0xff, 0xef, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xdf, 0xff,
  0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xbf, 0xff, 0xff, 0xe3, 0xe3, 0xe3,
  0xff, 0xff, 0x7f, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xfe,
  0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xe3, 0xe3, 0xe3,
  0xff, 0xff, 0xff, 0xfb, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xf7,
  0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xef, 0xff, 0xe3, 0xe3, 0xe3,
  0xff, 0xff, 0xff, 0xdf, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xbf,
  0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0x7f, 0xff, 0xe3, 0xe3, 0xe3,
  0xff, 0xff, 0xff, 0xff, 0xfe, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff,
  0xfd, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xe3, 0xe3, 0xe3,
  0xff, 0xff, 0xff, 0xff, 0xf7, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff,
  0xef, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xdf, 0xe3, 0xe3, 0xe3,
  0xff, 0xff, 0xff, 0xff, 0xbf, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff,
  0x7f, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe2, 0xe3, 0xe3,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xe1, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe3,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xc3, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xff, 0xa3, 0xe3, 0xe3,
  0xff, 0xff, 0xff, 0xff, 0xff, 0x63, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xe3, 0xe2, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xe1, 0xe3,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe3,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xc3, 0xe3, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xe3, 0xa3, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe3, 0x63, 0xe3,
  0xff, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xff, 0xfd, 0xff, 0xff,
  0xff, 0xff, 0xfc, 0xfc, 0xff, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc,
  0xff, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xff, 0xef, 0xff, 0xff,
  0xff, 0xff, 0xfc, 0xfc, 0xff, 0xdf, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc,
  0xff, 0xbf, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xff, 0x7f, 0xff, 0xff,
  0xff, 0xff, 0xfc, 0xfc, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xfc, 0xfc,
  0xff, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xff, 0xff, 0xfb, 0xff,
  0xff, 0xff, 0xfc, 0xfc, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xfc, 0xfc,
  0xff, 0xff, 0xef, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xff, 0xff, 0xdf, 0xff,
  0xff, 0xff, 0xfc, 0xfc, 0xff, 0xff, 0xbf, 0xff, 0xff, 0xff, 0xfc, 0xfc,
  0xff, 0xff, 0x7f, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xfe,
  0xff, 0xff, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xfc, 0xfc,
  0xff, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xf7,
  0xff, 0xff, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xef, 0xff, 0xff, 0xfc, 0xfc,
  0xff, 0xff, 0xff, 0xdf, 0xff, 0xff, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xbf,
  0xff, 0xff, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0x7f, 0xff, 0xff, 0xfc, 0xfc,
  0xff, 0xff, 0xff, 0xff, 0xfe, 0xff, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff,
  0xfd, 0xff, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xfc, 0xfc,
  0xff, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff,
  0xef, 0xff, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xdf, 0xff, 0xfc, 0xfc,
  0xff, 0xff, 0xff, 0xff, 0xbf, 0xff, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff,
  0x7f, 0xff, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xfc, 0xfc,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xfb, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xfc, 0xfc,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xef, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xdf, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xbf, 0xfc, 0xfc,
  0xff, 0xff, 0xff, 0xff, 0xff, 0x7f, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xfc, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xf4, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xec, 0xfc,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xdc, 0xfc, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xbc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x7c, 0xfc,
  0xff, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xff, 0xfd, 0xff, 0xff,
  0xff, 0xff, 0xf8, 0xf8, 0xff, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8,
  0xff, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xff, 0xef, 0xff, 0xff,
  0xff, 0xff, 0xf8, 0xf8, 0xff, 0xdf, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8,
  0xff, 0xbf, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xff, 0x7f, 0xff, 0xff,
  0xff, 0xff, 0xf8, 0xf8, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xf8, 0xf8,
  0xff, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xff, 0xff, 0xfb, 0xff,
  0xff, 0xff, 0xf8, 0xf8, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xf8, 0xf8,
  0xff, 0xff, 0xef, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xff, 0xff, 0xdf, 0xff,
  0xff, 0xff, 0xf8, 0xf8, 0xff, 0xff, 0xbf, 0xff, 0xff, 0xff, 0xf8, 0xf8,
  0xff, 0xff, 0x7f, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xfe,
  0xff, 0xff, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xf8, 0xf8,
  0xff, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xf7,
  0xff, 0xff, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xef, 0xff, 0xff, 0xf8, 0xf8,
  0xff, 0xff, 0xff, 0xdf, 0xff, 0xff, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xbf,
  0xff, 0xff, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0x7f, 0xff, 0xff, 0xf8, 0xf8,
  0xff, 0xff, 0xff, 0xff, 0xfe, 0xff, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff,
  0xfd, 0xff, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xf8, 0xf8,
  0xff, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff,
  0xef, 0xff, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xdf, 0xff, 0xf8, 0xf8,
  0xff, 0xff, 0xff, 0xff, 0xbf, 0xff, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff,
  0x7f, 0xff, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xf8, 0xf8,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xfb, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xf8, 0xf8,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xef, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xdf, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xbf, 0xf8, 0xf8,
  0xff, 0xff, 0xff, 0xff, 0xff, 0x7f, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xf0, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe8, 0xf8,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xd8, 0xf8, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xb8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x78, 0xf8,
  0xff, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xff, 0xfd, 0xff, 0xff,
  0xff, 0xff, 0xf1, 0xf1, 0xff, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1,
  0xff, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xff, 0xef, 0xff, 0xff,
  0xff, 0xff, 0xf1, 0xf1, 0xff, 0xdf, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1,
  0xff, 0xbf, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xff, 0x7f, 0xff, 0xff,
  0xff, 0xff, 0xf1, 0xf1, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xf1, 0xf1,
  0xff, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xff, 0xff, 0xfb, 0xff,
  0xff, 0xff, 0xf1, 0xf1, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xf1, 0xf1,
  0xff, 0xff, 0xef, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xff, 0xff, 0xdf, 0xff,
  0xff, 0xff, 0xf1, 0xf1, 0xff, 0xff, 0xbf, 0xff, 0xff, 0xff, 0xf1, 0xf1,
  0xff, 0xff, 0x7f, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xfe,
  0xff, 0xff, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xf1, 0xf1,
  0xff, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xf7,
  0xff, 0xff, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xef, 0xff, 0xff, 0xf1, 0xf1,
  0xff, 0xff, 0xff, 0xdf, 0xff, 0xff, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xbf,
  0xff, 0xff, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0x7f, 0xff, 0xff, 0xf1, 0xf1,
  0xff, 0xff, 0xff, 0xff, 0xfe, 0xff, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff,
  0xfd, 0xff, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xf1, 0xf1,
  0xff, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff,
  0xef, 0xff, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xdf, 0xff, 0xf1, 0xf1,
  0xff, 0xff, 0xff, 0xff, 0xbf, 0xff, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff,
  0x7f, 0xff, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xf1, 0xf1,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xfb, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xf1, 0xf1,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xef, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xdf, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xff, 0xbf, 0xf1, 0xf1,
  0xff, 0xff, 0xff, 0xff, 0xff, 0x7f, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xf0, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe1, 0xf1,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xd1, 0xf1, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xb1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x71, 0xf1,
  0xff, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xff, 0xfd, 0xff, 0xff,
  0xff, 0xff, 0xe3, 0xe3, 0xff, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xe3,
  0xff, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xff, 0xef, 0xff, 0xff,
  0xff, 0xff, 0xe3, 0xe3, 0xff, 0xdf, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xe3,
  0xff, 0xbf, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xff, 0x7f, 0xff, 0xff,
  0xff, 0xff, 0xe3, 0xe3, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xff, 0xe3, 0xe3,
  0xff, 0xff, 0xfd, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xff, 0xff, 0xfb, 0xff,
  0xff, 0xff, 0xe3, 0xe3, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xe3, 0xe3,
  0xff, 0xff, 0xef, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xff, 0xff, 0xdf, 0xff,
  0xff, 0xff, 0xe3, 0xe3, 0xff, 0xff, 0xbf, 0xff, 0xff, 0xff, 0xe3, 0xe3,
  0xff, 0xff, 0x7f, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xfe,
  0xff, 0xff, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xe3, 0xe3,
  0xff, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xf7,
  0xff, 0xff, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xef, 0xff, 0xff, 0xe3, 0xe3,
  0xff, 0xff, 0xff, 0xdf, 0xff, 0xff, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xbf,
  0xff, 0xff, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0x7f, 0xff, 0xff, 0xe3, 0xe3,
  0xff, 0xff, 0xff, 0xff, 0xfe, 0xff, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff,
  0xfd, 0xff, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xe3, 0xe3,
  0xff, 0xff, 0xff, 0xff, 0xf7, 0xff, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff,
  0xef, 0xff, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xdf, 0xff, 0xe3, 0xe3,
  0xff, 0xff, 0xff, 0xff, 0xbf, 0xff, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff,
  0x7f, 0xff, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xe3, 0xe3,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xfb, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xe3, 0xe3,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xef, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xdf, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xff, 0xbf, 0xe3, 0xe3,
  0xff, 0xff, 0xff, 0xff, 0xff, 0x7f, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xe2, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe1, 0xe3,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xe3,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xc3, 0xe3, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xa3, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x63, 0xe3,
  0x00, 0x00, 0x07, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x04, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x08, 0x14, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x2c, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x20, 0x54, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x40, 0xa4, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x44, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x04, 0x06, 0x07, 0x07, 0x00, 0x00, 0x00,
  0x08, 0x08, 0x04, 0x06, 0x00, 0x00, 0x00, 0x00, 0x1c, 0x14, 0x1b, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x7c, 0x68, 0x77, 0x7e, 0x00, 0x00, 0x00, 0x00,
  0xfc, 0xd4, 0xef, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xac, 0xdf, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xfc, 0x5c, 0xbf, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfc, 0xbc, 0x7f, 0xff, 0xff, 0xff, 0xff, 0xff, 0x1c, 0x1c, 0x1d, 0x0e,
  0x0f, 0x0f, 0x00, 0x00, 0x1c, 0x1c, 0x1a, 0x1d, 0x0f, 0x00, 0x00, 0x00,
  0x3c, 0x3c, 0x35, 0x3b, 0x00, 0x00, 0x00, 0x00, 0xfc, 0xfc, 0xeb, 0xf7,
  0xff, 0xfc, 0x00, 0x00, 0xfc, 0xfc, 0xd7, 0xef, 0xff, 0xff, 0xff, 0xff,
  0xfc, 0xfc, 0xaf, 0xdf, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0x5f, 0xbf,
  0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xbf, 0x7f, 0xff, 0xff, 0xff, 0xff,
  0xfc, 0xfc, 0xff, 0xfd, 0xfe, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xff, 0xfa,
  0xfd, 0xff, 0xff, 0xfd, 0xfc, 0xfc, 0xff, 0xf5, 0xfb, 0xff, 0xff, 0xff,
  0xfc, 0xfc, 0xff, 0xeb, 0xf7, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xff, 0xd7,
  0xef, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xff, 0xaf, 0xdf, 0xff, 0xff, 0xff,
  0xfc, 0xfc, 0xff, 0x5f, 0xbf, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xff, 0xbf,
  0x7f, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xff, 0xff, 0xfd, 0xfe, 0xff, 0xff,
  0xfc, 0xfc, 0xff, 0xff, 0xfa, 0xfd, 0xff, 0xff, 0xfc, 0xfc, 0xff, 0xff,
  0xf5, 0xfb, 0xff, 0xff, 0xfc, 0xfc, 0xff, 0xff, 0xeb, 0xf7, 0xff, 0xff,
  0xfc, 0xfc, 0xff, 0xff, 0xd7, 0xef, 0xff, 0xff, 0xfc, 0xfc, 0xff, 0xff,
  0xaf, 0xdf, 0xff, 0xff, 0xfc, 0xfc, 0xff, 0xff, 0x5f, 0xbf, 0xff, 0xff,
  0xfc, 0xfc, 0xff, 0xff, 0xbf, 0x7f, 0xff, 0xff, 0xfc, 0xfc, 0xff, 0xff,
  0xff, 0xfd, 0xfe, 0xff, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xfa, 0xfd, 0xff,
  0xfc, 0xfc, 0xff, 0xff, 0xff, 0xf5, 0xfb, 0xff, 0xfc, 0xfc, 0xff, 0xff,
  0xff, 0xeb, 0xf7, 0xff, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xd7, 0xef, 0xff,
  0xfc, 0xfc, 0xff, 0xff, 0xff, 0xaf, 0xdf, 0xff, 0xfc, 0xfc, 0xff, 0xff,
  0xff, 0x5f, 0xbf, 0xff, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xbf, 0x7f, 0xff,
  0x00, 0x00, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x08, 0x08, 0x06, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x08, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x28, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x20, 0x50, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x40, 0xa0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x40, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x06, 0x07, 0x00, 0x00, 0x00, 0x00,
  0x18, 0x08, 0x0d, 0x07, 0x00, 0x00, 0x00, 0x00, 0x10, 0x10, 0x08, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x38, 0x28, 0x36, 0x00, 0x00, 0x00, 0x00, 0x00,
  0xf8, 0xd0, 0xee, 0xfc, 0x00, 0x00, 0x00, 0x00, 0xf8, 0xa8, 0xdf, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xf8, 0x58, 0xbf, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xf8, 0xb8, 0x7f, 0xff, 0xff, 0xff, 0xff, 0xff, 0x08, 0x08, 0x0d, 0x0e,
  0x0f, 0x00, 0x00, 0x00, 0x38, 0x38, 0x3a, 0x1d, 0x1f, 0x00, 0x00, 0x00,
  0x38, 0x38, 0x35, 0x3a, 0x00, 0x00, 0x00, 0x00, 0x78, 0x78, 0x6b, 0x77,
  0x00, 0x00, 0x00, 0x00, 0xf8, 0xf8, 0xd7, 0xef, 0xfe, 0xf8, 0x00, 0x00,
  0xf8, 0xf8, 0xaf, 0xdf, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0x5f, 0xbf,
  0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xbf, 0x7f, 0xff, 0xff, 0xff, 0xff,
  0xf8, 0xf8, 0xff, 0xfd, 0xfe, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xff, 0xfa,
  0xfd, 0xff, 0xfd, 0xfd, 0xf8, 0xf8, 0xff, 0xf5, 0xfb, 0xff, 0xfb, 0xfb,
  0xf8, 0xf8, 0xff, 0xeb, 0xf7, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xff, 0xd7,
  0xef, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xff, 0xaf, 0xdf, 0xff, 0xff, 0xff,
  0xf8, 0xf8, 0xff, 0x5f, 0xbf, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xff, 0xbf,
  0x7f, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xff, 0xff, 0xfd, 0xfe, 0xff, 0xff,
  0xf8, 0xf8, 0xff, 0xff, 0xfa, 0xfd, 0xff, 0xff, 0xf8, 0xf8, 0xff, 0xff,
  0xf5, 0xfb, 0xff, 0xff, 0xf8, 0xf8, 0xff, 0xff, 0xeb, 0xf7, 0xff, 0xff,
  0xf8, 0xf8, 0xff, 0xff, 0xd7, 0xef, 0xff, 0xff, 0xf8, 0xf8, 0xff, 0xff,
  0xaf, 0xdf, 0xff, 0xff, 0xf8, 0xf8, 0xff, 0xff, 0x5f, 0xbf, 0xff, 0xff,
  0xf8, 0xf8, 0xff, 0xff, 0xbf, 0x7f, 0xff, 0xff, 0xf8, 0xf8, 0xff, 0xff,
  0xff, 0xfd, 0xfe, 0xff, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xfa, 0xfd, 0xff,
  0xf8, 0xf8, 0xff, 0xff, 0xff, 0xf5, 0xfb, 0xff, 0xf8, 0xf8, 0xff, 0xff,
  0xff, 0xeb, 0xf7, 0xff, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xd7, 0xef, 0xff,
  0xf8, 0xf8, 0xff, 0xff, 0xff, 0xaf, 0xdf, 0xff, 0xf8, 0xf8, 0xff, 0xff,
  0xff, 0x5f, 0xbf, 0xff, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xbf, 0x7f, 0xff,
  0x01, 0x00, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x11, 0x11, 0x0c, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x20, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x20, 0x50, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x40, 0xa0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x40, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x06, 0x07, 0x00, 0x00, 0x00, 0x00,
  0x10, 0x10, 0x0d, 0x00, 0x00, 0x00, 0x00, 0x00, 0x11, 0x11, 0x1b, 0x0c,
  0x00, 0x00, 0x00, 0x00, 0x21, 0x21, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x70, 0x50, 0x6c, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf0, 0xa0, 0xdc, 0xf8,
  0x00, 0x00, 0x00, 0x00, 0xf1, 0x51, 0xbf, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xf1, 0xb1, 0x7f, 0xff, 0xff, 0xff, 0xff, 0xff, 0x01, 0x01, 0x0d, 0x0e,
  0x0f, 0x00, 0x00, 0x00, 0x31, 0x31, 0x3a, 0x1d, 0x00, 0x00, 0x00, 0x00,
  0x71, 0x71, 0x35, 0x3b, 0x3e, 0x00, 0x00, 0x00, 0x71, 0x71, 0x6b, 0x74,
  0x00, 0x00, 0x00, 0x00, 0xf0, 0xf0, 0xd6, 0xee, 0x00, 0x00, 0x00, 0x00,
  0xf1, 0xf1, 0xaf, 0xdf, 0xfc, 0xf0, 0x00, 0x00, 0xf1, 0xf1, 0x5f, 0xbf,
  0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xbf, 0x7f, 0xff, 0xff, 0xff, 0xff,
  0xf1, 0xf1, 0xff, 0xfd, 0xfe, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xff, 0xfa,
  0xfd, 0xff, 0xfd, 0xfd, 0xf1, 0xf1, 0xff, 0xf5, 0xfb, 0xff, 0xfb, 0xfb,
  0xf1, 0xf1, 0xff, 0xeb, 0xf7, 0xff, 0xf7, 0xf7, 0xf1, 0xf1, 0xff, 0xd7,
  0xef, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xff, 0xaf, 0xdf, 0xff, 0xff, 0xff,
  0xf1, 0xf1, 0xff, 0x5f, 0xbf, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xff, 0xbf,
  0x7f, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xff, 0xff, 0xfd, 0xfe, 0xff, 0xff,
  0xf1, 0xf1, 0xff, 0xff, 0xfa, 0xfd, 0xff, 0xff, 0xf1, 0xf1, 0xff, 0xff,
  0xf5, 0xfb, 0xff, 0xff, 0xf1, 0xf1, 0xff, 0xff, 0xeb, 0xf7, 0xff, 0xff,
  0xf1, 0xf1, 0xff, 0xff, 0xd7, 0xef, 0xff, 0xff, 0xf1, 0xf1, 0xff, 0xff,
  0xaf, 0xdf, 0xff, 0xff, 0xf1, 0xf1, 0xff, 0xff, 0x5f, 0xbf, 0xff, 0xff,
  0xf1, 0xf1, 0xff, 0xff, 0xbf, 0x7f, 0xff, 0xff, 0xf1, 0xf1, 0xff, 0xff,
  0xff, 0xfd, 0xfe, 0xff, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xfa, 0xfd, 0xff,
  0xf1, 0xf1, 0xff, 0xff, 0xff, 0xf5, 0xfb, 0xff, 0xf1, 0xf1, 0xff, 0xff,
  0xff, 0xeb, 0xf7, 0xff, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xd7, 0xef, 0xff,
  0xf1, 0xf1, 0xff, 0xff, 0xff, 0xaf, 0xdf, 0xff, 0xf1, 0xf1, 0xff, 0xff,
  0xff, 0x5f, 0xbf, 0xff, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xbf, 0x7f, 0xff,
  0x01, 0x02, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x01, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x22, 0x22, 0x18, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x20, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x40, 0xa0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x42, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x03, 0x01, 0x0e, 0x0f, 0x0f, 0x00, 0x00, 0x00,
  0x03, 0x02, 0x0d, 0x00, 0x00, 0x00, 0x00, 0x00, 0x21, 0x21, 0x1b, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x22, 0x63, 0x36, 0x08, 0x00, 0x00, 0x00, 0x00,
  0x42, 0x42, 0x28, 0x00, 0x00, 0x00, 0x00, 0x00, 0xe0, 0xa0, 0xd8, 0x00,
  0x00, 0x00, 0x00, 0x00, 0xe0, 0x40, 0xb8, 0xf0, 0x00, 0x00, 0x00, 0x00,
  0xe3, 0xa3, 0x7f, 0xff, 0xff, 0xff, 0xff, 0xff, 0x23, 0x23, 0x3d, 0x3e,
  0x1f, 0x1f, 0x1a, 0x00, 0x03, 0x03, 0x1a, 0x1d, 0x00, 0x00, 0x00, 0x00,
  0x63, 0x63, 0x75, 0x3b, 0x00, 0x00, 0x00, 0x00, 0xe3, 0xe3, 0xeb, 0x77,
  0x1c, 0x00, 0x00, 0x00, 0xe3, 0xe3, 0xd7, 0xec, 0x00, 0x00, 0x00, 0x00,
  0xe0, 0xe0, 0xac, 0xdc, 0x00, 0x00, 0x00, 0x00, 0xe2, 0xe2, 0x5e, 0xbe,
  0xf8, 0xe0, 0x00, 0x00, 0xe3, 0xe3, 0xbf, 0x7f, 0xff, 0xff, 0xff, 0xff,
  0xe3, 0xe3, 0xff, 0xfd, 0xfe, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xff, 0xfa,
  0xfd, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xff, 0xf5, 0xfb, 0xff, 0xfb, 0xfb,
  0xe3, 0xe3, 0xff, 0xeb, 0xf7, 0xff, 0xf7, 0xf7, 0xe3, 0xe3, 0xff, 0xd7,
  0xef, 0xff, 0xef, 0xef, 0xe3, 0xe3, 0xff, 0xaf, 0xdf, 0xff, 0xff, 0xff,
  0xe3, 0xe3, 0xff, 0x5f, 0xbf, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xff, 0xbf,
  0x7f, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xff, 0xff, 0xfd, 0xfe, 0xff, 0xff,
  0xe3, 0xe3, 0xff, 0xff, 0xfa, 0xfd, 0xff, 0xff, 0xe3, 0xe3, 0xff, 0xff,
  0xf5, 0xfb, 0xff, 0xff, 0xe3, 0xe3, 0xff, 0xff, 0xeb, 0xf7, 0xff, 0xff,
  0xe3, 0xe3, 0xff, 0xff, 0xd7, 0xef, 0xff, 0xff, 0xe3, 0xe3, 0xff, 0xff,
  0xaf, 0xdf, 0xff, 0xff, 0xe3, 0xe3, 0xff, 0xff, 0x5f, 0xbf, 0xff, 0xff,
  0xe3, 0xe3, 0xff, 0xff, 0xbf, 0x7f, 0xff, 0xff, 0xe3, 0xe3, 0xff, 0xff,
  0xff, 0xfd, 0xfe, 0xff, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xfa, 0xfd, 0xff,
  0xe3, 0xe3, 0xff, 0xff, 0xff, 0xf5, 0xfb, 0xff, 0xe3, 0xe3, 0xff, 0xff,
  0xff, 0xeb, 0xf7, 0xff, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xd7, 0xef, 0xff,
  0xe3, 0xe3, 0xff, 0xff, 0xff, 0xaf, 0xdf, 0xff, 0xe3, 0xe3, 0xff, 0xff,
  0xff, 0x5f, 0xbf, 0xff, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xbf, 0x7f, 0xff,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x04, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x08, 0x14, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x2c, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x20, 0x54, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x40, 0xa4, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x44, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x0c, 0x0c, 0x04, 0x07, 0x07, 0x00, 0x00, 0x00,
  0x08, 0x08, 0x04, 0x06, 0x00, 0x00, 0x00, 0x00, 0x1c, 0x14, 0x18, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x7c, 0x68, 0x74, 0x7e, 0x00, 0x00, 0x00, 0x00,
  0xfc, 0xd4, 0xec, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xac, 0xdc, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xfc, 0x5c, 0xbc, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfc, 0xbc, 0x7c, 0xff, 0xff, 0xff, 0xff, 0xff, 0x1c, 0x1c, 0x1c, 0x0e,
  0x0f, 0x0f, 0x00, 0x00, 0x1c, 0x1c, 0x18, 0x0d, 0x0f, 0x00, 0x00, 0x00,
  0x3c, 0x3c, 0x34, 0x3b, 0x00, 0x00, 0x00, 0x00, 0xfc, 0xfc, 0xe8, 0xf7,
  0xfe, 0xfc, 0x00, 0x00, 0xfc, 0xfc, 0xd4, 0xef, 0xff, 0xff, 0xff, 0xff,
  0xfc, 0xfc, 0xac, 0xdf, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0x5c, 0xbf,
  0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xbc, 0x7f, 0xff, 0xff, 0xff, 0xff,
  0x3c, 0x3c, 0x3c, 0x3d, 0x1e, 0x1f, 0x1f, 0x00, 0x7c, 0x7c, 0x7c, 0x7a,
  0x3d, 0x1f, 0x00, 0x00, 0xfc, 0xfc, 0xfc, 0xf5, 0xfb, 0xfe, 0x00, 0x00,
  0xfc, 0xfc, 0xfc, 0xeb, 0xf7, 0xff, 0xfe, 0xfe, 0xfc, 0xfc, 0xfc, 0xd7,
  0xef, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xaf, 0xdf, 0xff, 0xff, 0xff,
  0xfc, 0xfc, 0xfc, 0x5f, 0xbf, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xbf,
  0x7f, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xfd, 0xfe, 0xbf, 0xbf,
  0xfc, 0xfc, 0xfc, 0xff, 0xfa, 0xfd, 0xff, 0x7f, 0xfc, 0xfc, 0xfc, 0xff,
  0xf5, 0xfb, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xeb, 0xf7, 0xff, 0xff,
  0xfc, 0xfc, 0xfc, 0xff, 0xd7, 0xef, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xff,
  0xaf, 0xdf, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0x5f, 0xbf, 0xff, 0xff,
  0xfc, 0xfc, 0xfc, 0xff, 0xbf, 0x7f, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xff,
  0xff, 0xfd, 0xbe, 0xbf, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xfa, 0xfd, 0x7f,
  0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xf5, 0xfb, 0xff, 0xfc, 0xfc, 0xfc, 0xff,
  0xff, 0xeb, 0xf7, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xd7, 0xef, 0xff,
  0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xaf, 0xdf, 0xff, 0xfc, 0xfc, 0xfc, 0xff,
  0xff, 0x5f, 0xbf, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xbf, 0x7f, 0xff,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x08, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x28, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x20, 0x50, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x40, 0xa0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x40, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x03, 0x00, 0x00, 0x00, 0x00,
  0x18, 0x18, 0x08, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x10, 0x10, 0x08, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x38, 0x28, 0x30, 0x00, 0x00, 0x00, 0x00, 0x00,
  0xf8, 0xd0, 0xe8, 0xfc, 0x00, 0x00, 0x00, 0x00, 0xf8, 0xa8, 0xd8, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xf8, 0x58, 0xb8, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xf8, 0xb8, 0x78, 0xff, 0xff, 0xff, 0xff, 0xff, 0x08, 0x08, 0x00, 0x06,
  0x07, 0x00, 0x00, 0x00, 0x38, 0x38, 0x38, 0x1d, 0x0f, 0x00, 0x00, 0x00,
  0x38, 0x38, 0x30, 0x18, 0x00, 0x00, 0x00, 0x00, 0x78, 0x78, 0x68, 0x76,
  0x00, 0x00, 0x00, 0x00, 0xf8, 0xf8, 0xd0, 0xef, 0xfc, 0xf8, 0x00, 0x00,
  0xf8, 0xf8, 0xa8, 0xdf, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0x58, 0xbf,
  0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xb8, 0x7f, 0xff, 0xff, 0xff, 0xff,
  0x38, 0x38, 0x38, 0x3d, 0x0e, 0x0f, 0x00, 0x00, 0x78, 0x78, 0x78, 0x7a,
  0x3d, 0x1f, 0x00, 0x00, 0xf8, 0xf8, 0xf8, 0xf5, 0x7a, 0x08, 0x00, 0x00,
  0xf8, 0xf8, 0xf8, 0xeb, 0xf7, 0xfc, 0x00, 0x00, 0xf8, 0xf8, 0xf8, 0xd7,
  0xef, 0xff, 0xfc, 0xfc, 0xf8, 0xf8, 0xf8, 0xaf, 0xdf, 0xff, 0xff, 0xff,
  0xf8, 0xf8, 0xf8, 0x5f, 0xbf, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xbf,
  0x7f, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xfd, 0xfe, 0xbf, 0xbf,
  0xf8, 0xf8, 0xf8, 0xff, 0xfa, 0xfd, 0x3f, 0x0f, 0xf8, 0xf8, 0xf8, 0xff,
  0xf5, 0xfb, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xeb, 0xf7, 0xff, 0xff,
  0xf8, 0xf8, 0xf8, 0xff, 0xd7, 0xef, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xff,
  0xaf, 0xdf, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0x5f, 0xbf, 0xff, 0xff,
  0xf8, 0xf8, 0xf8, 0xff, 0xbf, 0x7f, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xff,
  0xff, 0xfd, 0xbe, 0xbf, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xfa, 0x3d, 0x0f,
  0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xf5, 0xfb, 0xff, 0xf8, 0xf8, 0xf8, 0xff,
  0xff, 0xeb, 0xf7, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xd7, 0xef, 0xff,
  0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xaf, 0xdf, 0xff, 0xf8, 0xf8, 0xf8, 0xff,
  0xff, 0x5f, 0xbf, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xbf, 0x7f, 0xff,
  0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x20, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x20, 0x50, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x40, 0xa0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x40, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x00, 0x07, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x31, 0x31, 0x11, 0x1c,
  0x00, 0x00, 0x00, 0x00, 0x20, 0x20, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x70, 0x50, 0x60, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf0, 0xa0, 0xd0, 0xf8,
  0x00, 0x00, 0x00, 0x00, 0xf1, 0x51, 0xb1, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xf1, 0xb1, 0x71, 0xff, 0xff, 0xff, 0xff, 0xff, 0x01, 0x01, 0x01, 0x0e,
  0x0f, 0x00, 0x00, 0x00, 0x11, 0x11, 0x00, 0x0d, 0x00, 0x00, 0x00, 0x00,
  0x71, 0x71, 0x71, 0x3b, 0x0c, 0x00, 0x00, 0x00, 0x71, 0x71, 0x60, 0x30,
  0x00, 0x00, 0x00, 0x00, 0xf0, 0xf0, 0xd0, 0xec, 0x00, 0x00, 0x00, 0x00,
  0xf1, 0xf1, 0xa1, 0xdf, 0xf8, 0xf0, 0x00, 0x00, 0xf1, 0xf1, 0x51, 0xbf,
  0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xb1, 0x7f, 0xff, 0xff, 0xff, 0xff,
  0x31, 0x31, 0x31, 0x3d, 0x1e, 0x1f, 0x00, 0x00, 0x71, 0x71, 0x71, 0x7a,
  0x1d, 0x01, 0x00, 0x00, 0xf1, 0xf1, 0xf1, 0xf5, 0x7b, 0x3e, 0x00, 0x00,
  0xf1, 0xf1, 0xf1, 0xeb, 0xf4, 0x10, 0x00, 0x00, 0xf1, 0xf1, 0xf1, 0xd7,
  0xee, 0xf8, 0x00, 0x00, 0xf1, 0xf1, 0xf1, 0xaf, 0xdf, 0xff, 0xf8, 0xf8,
  0xf1, 0xf1, 0xf1, 0x5f, 0xbf, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xbf,
  0x7f, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xfd, 0xfe, 0xff, 0xff,
  0xf1, 0xf1, 0xf1, 0xff, 0xfa, 0xfd, 0x3f, 0x0f, 0xf1, 0xf1, 0xf1, 0xff,
  0xf5, 0xfb, 0xff, 0x1f, 0xf1, 0xf1, 0xf1, 0xff, 0xeb, 0xf7, 0xff, 0xfe,
  0xf1, 0xf1, 0xf1, 0xff, 0xd7, 0xef, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xff,
  0xaf, 0xdf, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0x5f, 0xbf, 0xff, 0xff,
  0xf1, 0xf1, 0xf1, 0xff, 0xbf, 0x7f, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xff,
  0xff, 0xfd, 0xfe, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xfa, 0x3d, 0x0f,
  0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xf5, 0xfb, 0x1f, 0xf1, 0xf1, 0xf1, 0xff,
  0xff, 0xeb, 0xf7, 0xfe, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xd7, 0xef, 0xff,
  0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xaf, 0xdf, 0xff, 0xf1, 0xf1, 0xf1, 0xff,
  0xff, 0x5f, 0xbf, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xbf, 0x7f, 0xff,
  0x01, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x01, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x20, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x40, 0xa0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x40, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x03, 0x01, 0x02, 0x0f, 0x07, 0x00, 0x00, 0x00,
  0x03, 0x02, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x03, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x63, 0x63, 0x22, 0x18, 0x00, 0x00, 0x00, 0x00,
  0x40, 0x40, 0x20, 0x00, 0x00, 0x00, 0x00, 0x00, 0xe0, 0xa0, 0xc0, 0x00,
  0x00, 0x00, 0x00, 0x00, 0xe0, 0x40, 0xa0, 0xf0, 0x00, 0x00, 0x00, 0x00,
  0xe3, 0xa3, 0x63, 0xff, 0xff, 0xff, 0xff, 0xff, 0x23, 0x23, 0x21, 0x3e,
  0x0f, 0x0f, 0x00, 0x00, 0x03, 0x03, 0x02, 0x0d, 0x00, 0x00, 0x00, 0x00,
  0x23, 0x23, 0x01, 0x1b, 0x00, 0x00, 0x00, 0x00, 0xe3, 0xe3, 0xe3, 0x77,
  0x18, 0x00, 0x00, 0x00, 0xe2, 0xe2, 0xc0, 0x68, 0x00, 0x00, 0x00, 0x00,
  0xe0, 0xe0, 0xa0, 0xd8, 0x00, 0x00, 0x00, 0x00, 0xe2, 0xe2, 0x42, 0xbe,
  0xf0, 0xe0, 0x00, 0x00, 0xe3, 0xe3, 0xa3, 0x7f, 0xff, 0xff, 0xff, 0xff,
  0xe3, 0xe3, 0xe3, 0xfd, 0xfe, 0xff, 0x1f, 0x1f, 0x63, 0x63, 0x63, 0x7a,
  0x1d, 0x07, 0x00, 0x00, 0xe3, 0xe3, 0xe3, 0xf5, 0x3b, 0x02, 0x00, 0x00,
  0xe3, 0xe3, 0xe3, 0xeb, 0xf7, 0x1c, 0x00, 0x00, 0xe3, 0xe3, 0xe3, 0xd7,
  0xec, 0x20, 0x00, 0x00, 0xe3, 0xe3, 0xe3, 0xaf, 0xdc, 0xf0, 0x00, 0x00,
  0xe3, 0xe3, 0xe3, 0x5f, 0xbf, 0xff, 0xf0, 0xf0, 0xe3, 0xe3, 0xe3, 0xbf,
  0x7f, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xfd, 0xfe, 0xff, 0xff,
  0xe3, 0xe3, 0xe3, 0xff, 0xfa, 0xfd, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xff,
  0xf5, 0xfb, 0x7f, 0x1f, 0xe3, 0xe3, 0xe3, 0xff, 0xeb, 0xf7, 0xff, 0x3e,
  0xe3, 0xe3, 0xe3, 0xff, 0xd7, 0xef, 0xff, 0xfc, 0xe3, 0xe3, 0xe3, 0xff,
  0xaf, 0xdf, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0x5f, 0xbf, 0xff, 0xff,
  0xe3, 0xe3, 0xe3, 0xff, 0xbf, 0x7f, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xff,
  0xff, 0xfd, 0xfe, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xfa, 0xfd, 0xff,
  0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xf5, 0x7b, 0x1f, 0xe3, 0xe3, 0xe3, 0xff,
  0xff, 0xeb, 0xf7, 0x3e, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xd7, 0xef, 0xfc,
  0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xaf, 0xdf, 0xff, 0xe3, 0xe3, 0xe3, 0xff,
  0xff, 0x5f, 0xbf, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xbf, 0x7f, 0xff,
  0x01, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x04, 0x04, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x04, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x08, 0x14, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x28, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x20, 0x50, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x40, 0xa0, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x40, 0x04, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x0f, 0x08, 0x0c, 0x0c, 0x00, 0x00, 0x00, 0x00, 0x1f, 0x14, 0x18, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x7f, 0x68, 0x74, 0x7c, 0x00, 0x00, 0x00, 0x00,
  0xff, 0xd4, 0xec, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xac, 0xdc, 0xfc,
  0xff, 0xff, 0xff, 0xff, 0xff, 0x5c, 0xbc, 0xfc, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xbc, 0x7c, 0xfc, 0xff, 0xff, 0xff, 0xff, 0x1f, 0x1c, 0x1c, 0x1c,
  0x0f, 0x07, 0x00, 0x00, 0x3f, 0x3c, 0x38, 0x1c, 0x1e, 0x00, 0x00, 0x00,
  0x3f, 0x3c, 0x34, 0x38, 0x00, 0x00, 0x00, 0x00, 0xff, 0xfc, 0xe8, 0xf4,
  0xfe, 0xfc, 0x00, 0x00, 0xff, 0xfc, 0xd4, 0xec, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xfc, 0xac, 0xdc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0x5c, 0xbc,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xbc, 0x7c, 0xff, 0xff, 0xff, 0xff,
  0x3f, 0x3c, 0x3c, 0x3c, 0x3e, 0x1f, 0x0f, 0x00, 0x7f, 0x7c, 0x7c, 0x78,
  0x3d, 0x3f, 0x00, 0x00, 0xff, 0xfc, 0xfc, 0xf4, 0xfb, 0xfe, 0x00, 0x00,
  0xff, 0xfc, 0xfc, 0xe8, 0xf7, 0xff, 0xfe, 0xfe, 0xff, 0xfc, 0xfc, 0xd4,
  0xef, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xac, 0xdf, 0xff, 0xff, 0xff,
  0xff, 0xfc, 0xfc, 0x5c, 0xbf, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xbc,
  0x7f, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xfd, 0xfe, 0x3f, 0x1f,
  0xff, 0xfc, 0xfc, 0xfc, 0xfa, 0xfd, 0x7f, 0x00, 0xff, 0xfc, 0xfc, 0xfc,
  0xf5, 0xfb, 0xff, 0x7f, 0xff, 0xfc, 0xfc, 0xfc, 0xeb, 0xf7, 0xff, 0xff,
  0xff, 0xfc, 0xfc, 0xfc, 0xd7, 0xef, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc,
  0xaf, 0xdf, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0x5f, 0xbf, 0xff, 0xff,
  0xff, 0xfc, 0xfc, 0xfc, 0xbf, 0x7f, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc,
  0xff, 0xfd, 0x3e, 0x1f, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xfa, 0x7d, 0x00,
  0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xf5, 0xfb, 0x7f, 0xff, 0xfc, 0xfc, 0xfc,
  0xff, 0xeb, 0xf7, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xd7, 0xef, 0xff,
  0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xaf, 0xdf, 0xff, 0xff, 0xfc, 0xfc, 0xfc,
  0xff, 0x5f, 0xbf, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xbf, 0x7f, 0xff,
  0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x04, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x08, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x28, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x20, 0x50, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x40, 0xa0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x40, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x07, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1f, 0x10, 0x18, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x3e, 0x28, 0x30, 0x00, 0x00, 0x00, 0x00, 0x00,
  0xfe, 0xd0, 0xe8, 0xf8, 0x00, 0x00, 0x00, 0x00, 0xff, 0xa8, 0xd8, 0xf8,
  0xff, 0xff, 0xff, 0xff, 0xff, 0x58, 0xb8, 0xf8, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xb8, 0x78, 0xf8, 0xff, 0xff, 0xff, 0xff, 0x1f, 0x18, 0x18, 0x00,
  0x07, 0x00, 0x00, 0x00, 0x3f, 0x38, 0x38, 0x38, 0x0f, 0x01, 0x00, 0x00,
  0x3f, 0x38, 0x30, 0x38, 0x00, 0x00, 0x00, 0x00, 0x7f, 0x78, 0x68, 0x70,
  0x00, 0x00, 0x00, 0x00, 0xff, 0xf8, 0xd0, 0xe8, 0xfc, 0xf8, 0x00, 0x00,
  0xff, 0xf8, 0xa8, 0xd8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0x58, 0xb8,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xb8, 0x78, 0xff, 0xff, 0xff, 0xff,
  0x3f, 0x38, 0x38, 0x38, 0x0e, 0x0f, 0x00, 0x00, 0x7f, 0x78, 0x78, 0x78,
  0x7d, 0x1f, 0x00, 0x00, 0xff, 0xf8, 0xf8, 0xf0, 0x78, 0x08, 0x00, 0x00,
  0xff, 0xf8, 0xf8, 0xe8, 0xf6, 0xfc, 0x00, 0x00, 0xff, 0xf8, 0xf8, 0xd0,
  0xef, 0xff, 0xfc, 0xfc, 0xff, 0xf8, 0xf8, 0xa8, 0xdf, 0xff, 0xff, 0xff,
  0xff, 0xf8, 0xf8, 0x58, 0xbf, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xb8,
  0x7f, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xfd, 0xfe, 0x1f, 0x00,
  0xff, 0xf8, 0xf8, 0xf8, 0xfa, 0xfd, 0x3f, 0x07, 0xff, 0xf8, 0xf8, 0xf8,
  0xf5, 0xfb, 0x1d, 0x00, 0xff, 0xf8, 0xf8, 0xf8, 0xeb, 0xf7, 0xfe, 0xfe,
  0xff, 0xf8, 0xf8, 0xf8, 0xd7, 0xef, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8,
  0xaf, 0xdf, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0x5f, 0xbf, 0xff, 0xff,
  0xff, 0xf8, 0xf8, 0xf8, 0xbf, 0x7f, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8,
  0xff, 0xfd, 0x1e, 0x00, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xfa, 0x3d, 0x07,
  0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xf5, 0x19, 0x00, 0xff, 0xf8, 0xf8, 0xf8,
  0xff, 0xeb, 0xf6, 0xfe, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xd7, 0xef, 0xff,
  0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xaf, 0xdf, 0xff, 0xff, 0xf8, 0xf8, 0xf8,
  0xff, 0x5f, 0xbf, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xbf, 0x7f, 0xff,
  0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x01, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x08, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x20, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x20, 0x50, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x40, 0xa0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x40, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x07, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00,
  0x0f, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x3e, 0x20, 0x30, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x7c, 0x50, 0x60, 0x00, 0x00, 0x00, 0x00, 0x00, 0xfc, 0xa0, 0xd0, 0xf0,
  0x00, 0x00, 0x00, 0x00, 0xff, 0x51, 0xb1, 0xf1, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xb1, 0x71, 0xf1, 0xff, 0xff, 0xff, 0xff, 0x0f, 0x01, 0x01, 0x00,
  0x07, 0x00, 0x00, 0x00, 0x1f, 0x11, 0x10, 0x01, 0x00, 0x00, 0x00, 0x00,
  0x7f, 0x71, 0x71, 0x71, 0x1c, 0x00, 0x00, 0x00, 0x7f, 0x71, 0x61, 0x70,
  0x00, 0x00, 0x00, 0x00, 0xfe, 0xf0, 0xd0, 0xe0, 0x00, 0x00, 0x00, 0x00,
  0xff, 0xf1, 0xa1, 0xd1, 0xf8, 0xf0, 0x00, 0x00, 0xff, 0xf1, 0x51, 0xb1,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xb1, 0x71, 0xff, 0xff, 0xff, 0xff,
  0x3f, 0x31, 0x31, 0x31, 0x0e, 0x0f, 0x00, 0x00, 0x7f, 0x71, 0x71, 0x70,
  0x0d, 0x01, 0x00, 0x00, 0xff, 0xf1, 0xf1, 0xf1, 0xfb, 0x1e, 0x00, 0x00,
  0xff, 0xf1, 0xf1, 0xe1, 0xf0, 0x10, 0x00, 0x00, 0xff, 0xf1, 0xf1, 0xd1,
  0xec, 0xf8, 0x00, 0x00, 0xff, 0xf1, 0xf1, 0xa1, 0xdf, 0xff, 0xf8, 0xf8,
  0xff, 0xf1, 0xf1, 0x51, 0xbf, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xb1,
  0x7f, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xfd, 0xfe, 0x1f, 0x1f,
  0xff, 0xf1, 0xf1, 0xf1, 0xfa, 0xfd, 0x0b, 0x01, 0xff, 0xf1, 0xf1, 0xf1,
  0xf5, 0xfb, 0x3f, 0x00, 0xff, 0xf1, 0xf1, 0xf1, 0xeb, 0xf7, 0x3a, 0x00,
  0xff, 0xf1, 0xf1, 0xf1, 0xd7, 0xef, 0xfc, 0xfc, 0xff, 0xf1, 0xf1, 0xf1,
  0xaf, 0xdf, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0x5f, 0xbf, 0xff, 0xff,
  0xff, 0xf1, 0xf1, 0xf1, 0xbf, 0x7f, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf1,
  0xff, 0xfd, 0x1e, 0x1f, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xfa, 0x09, 0x01,
  0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xf5, 0x3b, 0x00, 0xff, 0xf1, 0xf1, 0xf1,
  0xff, 0xeb, 0x32, 0x00, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xd7, 0xec, 0xfc,
  0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xaf, 0xdf, 0xff, 0xff, 0xf1, 0xf1, 0xf1,
  0xff, 0x5f, 0xbf, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xbf, 0x7f, 0xff,
  0x01, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x01, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x04, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x20, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x20, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x40, 0xa0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x40, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x0f, 0x01, 0x02, 0x03, 0x07, 0x00, 0x00, 0x00,
  0x0f, 0x02, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1f, 0x01, 0x03, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x7c, 0x40, 0x60, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf8, 0xa0, 0xc0, 0x00,
  0x00, 0x00, 0x00, 0x00, 0xf8, 0x40, 0xa0, 0xe0, 0x00, 0x00, 0x00, 0x00,
  0xff, 0xa3, 0x63, 0xe3, 0xff, 0xff, 0xff, 0xff, 0x3f, 0x23, 0x21, 0x22,
  0x0f, 0x07, 0x00, 0x00, 0x1f, 0x03, 0x02, 0x01, 0x00, 0x00, 0x00, 0x00,
  0x3f, 0x23, 0x21, 0x03, 0x00, 0x00, 0x00, 0x00, 0xff, 0xe3, 0xe3, 0xe3,
  0x18, 0x00, 0x00, 0x00, 0xfe, 0xe2, 0xc2, 0xe0, 0x00, 0x00, 0x00, 0x00,
  0xfc, 0xe0, 0xa0, 0xc0, 0x00, 0x00, 0x00, 0x00, 0xfe, 0xe2, 0x42, 0xa2,
  0xf0, 0xe0, 0x00, 0x00, 0xff, 0xe3, 0xa3, 0x63, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xe3, 0xe3, 0xe1, 0xfe, 0xff, 0x0f, 0x0f, 0x7f, 0x63, 0x63, 0x62,
  0x0d, 0x07, 0x00, 0x00, 0xff, 0xe3, 0xe3, 0xe1, 0x1b, 0x02, 0x00, 0x00,
  0xff, 0xe3, 0xe3, 0xe3, 0xf7, 0x1c, 0x00, 0x00, 0xff, 0xe3, 0xe3, 0xc3,
  0xe8, 0x20, 0x00, 0x00, 0xff, 0xe3, 0xe3, 0xa3, 0xd8, 0xf0, 0x00, 0x00,
  0xff, 0xe3, 0xe3, 0x43, 0xbf, 0xff, 0xf0, 0xf0, 0xff, 0xe3, 0xe3, 0xa3,
  0x7f, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xfd, 0xfe, 0xff, 0xff,
  0xff, 0xe3, 0xe3, 0xe3, 0xfa, 0xfd, 0x0f, 0x0f, 0xff, 0xe3, 0xe3, 0xe3,
  0xf5, 0xfb, 0x07, 0x00, 0xff, 0xe3, 0xe3, 0xe3, 0xeb, 0xf7, 0x7e, 0x00,
  0xff, 0xe3, 0xe3, 0xe3, 0xd7, 0xef, 0x74, 0x00, 0xff, 0xe3, 0xe3, 0xe3,
  0xaf, 0xdf, 0xf8, 0xf8, 0xff, 0xe3, 0xe3, 0xe3, 0x5f, 0xbf, 0xff, 0xff,
  0xff, 0xe3, 0xe3, 0xe3, 0xbf, 0x7f, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe3,
  0xff, 0xfd, 0xfe, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xfa, 0x0d, 0x0f,
  0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xf5, 0x13, 0x00, 0xff, 0xe3, 0xe3, 0xe3,
  0xff, 0xeb, 0x76, 0x00, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xd7, 0x64, 0x00,
  0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xaf, 0xd8, 0xf8, 0xff, 0xe3, 0xe3, 0xe3,
  0xff, 0x5f, 0xbf, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xbf, 0x7f, 0xff,
  0x01, 0x06, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x05, 0x04, 0x04,
  0x00, 0x00, 0x00, 0x00, 0x04, 0x0a, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x08, 0x14, 0x00, 0x04, 0x00, 0x00, 0x00, 0x00, 0x10, 0x28, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x20, 0x50, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x40, 0xa0, 0x00, 0x04, 0x00, 0x00, 0x00, 0x00, 0x80, 0x40, 0x00, 0x04,
  0x00, 0x00, 0x00, 0x00, 0x0f, 0x0d, 0x0c, 0x04, 0x00, 0x00, 0x00, 0x00,
  0x1f, 0x1a, 0x1c, 0x1c, 0x0c, 0x00, 0x00, 0x00, 0x3f, 0x35, 0x38, 0x3c,
  0x00, 0x00, 0x00, 0x00, 0x7f, 0x6b, 0x74, 0x7c, 0x00, 0x00, 0x00, 0x00,
  0xff, 0xd7, 0xec, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xaf, 0xdc, 0xfc,
  0xfc, 0xff, 0xff, 0xff, 0xff, 0x5f, 0xbc, 0xfc, 0xfc, 0xff, 0xff, 0xff,
  0xff, 0xbf, 0x7c, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x3f, 0x3f, 0x38, 0x3c, 0x3c, 0x0f, 0x00, 0x00,
  0x7f, 0x7f, 0x74, 0x78, 0x7c, 0x00, 0x00, 0x00, 0xff, 0xff, 0xe8, 0xf4,
  0xfc, 0xfc, 0x00, 0x00, 0xff, 0xff, 0xd4, 0xec, 0xfc, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xac, 0xdc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0x5c, 0xbc,
  0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xbc, 0x7c, 0xfc, 0xff, 0xff, 0xff,
  0x3f, 0x3f, 0x3c, 0x3c, 0x3c, 0x1f, 0x0f, 0x00, 0x7f, 0x7f, 0x7c, 0x78,
  0x7c, 0x7f, 0x1f, 0x00, 0xff, 0xff, 0xfc, 0xf4, 0xf8, 0xfe, 0x00, 0x00,
  0xff, 0xff, 0xfc, 0xe8, 0xf4, 0xff, 0xfe, 0xfe, 0xff, 0xff, 0xfc, 0xd4,
  0xec, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xac, 0xdc, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xfc, 0x5c, 0xbc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xbc,
  0x7c, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xfe, 0x7f, 0x1f,
  0xff, 0xff, 0xfc, 0xfc, 0xf8, 0xfd, 0xff, 0x7f, 0xff, 0xff, 0xfc, 0xfc,
  0xf4, 0xfb, 0xff, 0x7f, 0xff, 0xff, 0xfc, 0xfc, 0xe8, 0xf7, 0xff, 0xff,
  0xff, 0xff, 0xfc, 0xfc, 0xd4, 0xef, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc,
  0xac, 0xdf, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0x5c, 0xbf, 0xff, 0xff,
  0xff, 0xff, 0xfc, 0xfc, 0xbc, 0x7f, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc,
  0xfc, 0xfd, 0x7e, 0x1f, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xfa, 0xfd, 0x3f,
  0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xf5, 0xfb, 0x7f, 0xff, 0xff, 0xfc, 0xfc,
  0xfc, 0xeb, 0xf7, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xd7, 0xef, 0xff,
  0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xaf, 0xdf, 0xff, 0xff, 0xff, 0xfc, 0xfc,
  0xfc, 0x5f, 0xbf, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xbf, 0x7f, 0xff,
  0x01, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x05, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x04, 0x0a, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x08, 0x14, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x28, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x20, 0x50, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x40, 0xa0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x40, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x0f, 0x0d, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x1f, 0x1a, 0x08, 0x08, 0x00, 0x00, 0x00, 0x00, 0x3f, 0x35, 0x38, 0x38,
  0x00, 0x00, 0x00, 0x00, 0x7f, 0x6b, 0x70, 0x78, 0x00, 0x00, 0x00, 0x00,
  0xfe, 0xd6, 0xe8, 0xf8, 0x00, 0x00, 0x00, 0x00, 0xff, 0xaf, 0xd8, 0xf8,
  0xf8, 0xff, 0xff, 0xff, 0xff, 0x5f, 0xb8, 0xf8, 0xf8, 0xff, 0xff, 0xff,
  0xff, 0xbf, 0x78, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0x1f, 0x1f, 0x18, 0x18,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x7f, 0x7f, 0x70, 0x78, 0x78, 0x00, 0x00, 0x00, 0xff, 0xff, 0xe8, 0xf0,
  0xf8, 0x00, 0x00, 0x00, 0xff, 0xff, 0xd0, 0xe8, 0xf8, 0xe8, 0x00, 0x00,
  0xff, 0xff, 0xa8, 0xd8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0x58, 0xb8,
  0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xb8, 0x78, 0xf8, 0xff, 0xff, 0xff,
  0x3f, 0x3f, 0x38, 0x38, 0x38, 0x07, 0x00, 0x00, 0x7f, 0x7f, 0x78, 0x78,
  0x78, 0x1f, 0x03, 0x00, 0xff, 0xff, 0xf8, 0xf0, 0xf8, 0xfc, 0x00, 0x00,
  0xff, 0xff, 0xf8, 0xe8, 0xf0, 0xfc, 0x00, 0x00, 0xff, 0xff, 0xf8, 0xd0,
  0xe8, 0xff, 0xfc, 0xfc, 0xff, 0xff, 0xf8, 0xa8, 0xd8, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xf8, 0x58, 0xb8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xb8,
  0x78, 0xff, 0xff, 0xff, 0x7f, 0x7f, 0x78, 0x78, 0x78, 0x7e, 0x0f, 0x00,
  0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xfd, 0x7f, 0x0f, 0xff, 0xff, 0xf8, 0xf8,
  0xf0, 0xfb, 0xfe, 0x00, 0xff, 0xff, 0xf8, 0xf8, 0xe8, 0xf7, 0xfe, 0xfe,
  0xff, 0xff, 0xf8, 0xf8, 0xd0, 0xef, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8,
  0xa8, 0xdf, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0x58, 0xbf, 0xff, 0xff,
  0xff, 0xff, 0xf8, 0xf8, 0xb8, 0x7f, 0xff, 0xff, 0x7f, 0x7f, 0x78, 0x78,
  0x78, 0x7d, 0x0e, 0x00, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xfa, 0x3d, 0x0f,
  0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xf5, 0xfa, 0x00, 0xff, 0xff, 0xf8, 0xf8,
  0xf8, 0xeb, 0xf6, 0xfe, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xd7, 0xef, 0xff,
  0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xaf, 0xdf, 0xff, 0xff, 0xff, 0xf8, 0xf8,
  0xf8, 0x5f, 0xbf, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xbf, 0x7f, 0xff,
  0x01, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x05, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x04, 0x0a, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x08, 0x14, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x28, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x20, 0x50, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x40, 0xa0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x40, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x0f, 0x0d, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00,
  0x1f, 0x1a, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x3f, 0x35, 0x11, 0x11,
  0x00, 0x00, 0x00, 0x00, 0x7f, 0x6b, 0x70, 0x70, 0x00, 0x00, 0x00, 0x00,
  0xfe, 0xd6, 0xe0, 0xf0, 0x00, 0x00, 0x00, 0x00, 0xfc, 0xac, 0xd0, 0xf0,
  0x00, 0x00, 0x00, 0x00, 0xff, 0x5f, 0xb1, 0xf1, 0xf1, 0xff, 0xff, 0xff,
  0xff, 0xbf, 0x71, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0x1f, 0x1f, 0x11, 0x00,
  0x01, 0x00, 0x00, 0x00, 0x3f, 0x3f, 0x30, 0x11, 0x01, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xe1, 0xf1,
  0xf0, 0x00, 0x00, 0x00, 0xff, 0xff, 0xd1, 0xe0, 0xf0, 0x00, 0x00, 0x00,
  0xff, 0xff, 0xa1, 0xd1, 0xf0, 0xd0, 0x00, 0x00, 0xff, 0xff, 0x51, 0xb1,
  0xf1, 0xff, 0xff, 0xff, 0xff, 0xff, 0xb1, 0x71, 0xf1, 0xff, 0xff, 0xff,
  0x3f, 0x3f, 0x31, 0x31, 0x00, 0x07, 0x00, 0x00, 0x7f, 0x7f, 0x71, 0x70,
  0x31, 0x03, 0x00, 0x00, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0x3f, 0x04, 0x00,
  0xff, 0xff, 0xf1, 0xe1, 0xf1, 0xf8, 0x00, 0x00, 0xff, 0xff, 0xf1, 0xd1,
  0xe0, 0xf8, 0x00, 0x00, 0xff, 0xff, 0xf1, 0xa1, 0xd1, 0xff, 0xf8, 0xf8,
  0xff, 0xff, 0xf1, 0x51, 0xb1, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xb1,
  0x71, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xfe, 0x0f, 0x06,
  0xff, 0xff, 0xf1, 0xf1, 0xf0, 0xfd, 0x07, 0x01, 0xff, 0xff, 0xf1, 0xf1,
  0xf1, 0xfb, 0x7f, 0x1f, 0xff, 0xff, 0xf1, 0xf1, 0xe1, 0xf7, 0xfc, 0x00,
  0xff, 0xff, 0xf1, 0xf1, 0xd1, 0xef, 0xfc, 0xfc, 0xff, 0xff, 0xf1, 0xf1,
  0xa1, 0xdf, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0x51, 0xbf, 0xff, 0xff,
  0xff, 0xff, 0xf1, 0xf1, 0xb1, 0x7f, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1,
  0xf1, 0xfd, 0x0e, 0x0f, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xfa, 0x05, 0x01,
  0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xf5, 0x7b, 0x1f, 0xff, 0xff, 0xf1, 0xf1,
  0xf1, 0xeb, 0xf4, 0x00, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xd7, 0xec, 0xfc,
  0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xaf, 0xdf, 0xff, 0xff, 0xff, 0xf1, 0xf1,
  0xf1, 0x5f, 0xbf, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xbf, 0x7f, 0xff,
  0x01, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x05, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x04, 0x0a, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x08, 0x14, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x28, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x20, 0x50, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x40, 0xa0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x40, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x0f, 0x0d, 0x02, 0x03, 0x03, 0x00, 0x00, 0x00,
  0x1f, 0x1a, 0x01, 0x03, 0x00, 0x00, 0x00, 0x00, 0x3f, 0x35, 0x03, 0x03,
  0x00, 0x00, 0x00, 0x00, 0x7f, 0x6b, 0x22, 0x22, 0x00, 0x00, 0x00, 0x00,
  0xfe, 0xd6, 0xe0, 0xe0, 0x00, 0x00, 0x00, 0x00, 0xfc, 0xac, 0xc0, 0xe0,
  0x00, 0x00, 0x00, 0x00, 0xf8, 0x58, 0xa0, 0xe0, 0x00, 0x00, 0x00, 0x00,
  0xff, 0xbf, 0x63, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0x3f, 0x3f, 0x21, 0x22,
  0x03, 0x07, 0x00, 0x00, 0x3f, 0x3f, 0x22, 0x01, 0x03, 0x00, 0x00, 0x00,
  0x7f, 0x7f, 0x61, 0x23, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xc3, 0xe2, 0xe0, 0x00, 0x00, 0x00,
  0xfe, 0xfe, 0xa2, 0xc0, 0xe0, 0x00, 0x00, 0x00, 0xfe, 0xfe, 0x42, 0xa2,
  0xe0, 0xa0, 0x00, 0x00, 0xff, 0xff, 0xa3, 0x63, 0xe3, 0xff, 0xff, 0xff,
  0xff, 0xff, 0xe3, 0xe1, 0xe2, 0xff, 0x0f, 0x02, 0x7f, 0x7f, 0x63, 0x62,
  0x01, 0x07, 0x00, 0x00, 0xff, 0xff, 0xe3, 0xe1, 0x63, 0x07, 0x00, 0x00,
  0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0x08, 0x00, 0xff, 0xff, 0xe3, 0xc3,
  0xe3, 0xf0, 0x00, 0x00, 0xff, 0xff, 0xe3, 0xa3, 0xc0, 0xf0, 0x00, 0x00,
  0xff, 0xff, 0xe3, 0x43, 0xa3, 0xff, 0xf0, 0xf0, 0xff, 0xff, 0xe3, 0xa3,
  0x63, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe1, 0xfe, 0xff, 0xff,
  0xff, 0xff, 0xe3, 0xe3, 0xe2, 0xfd, 0x0f, 0x0f, 0xff, 0xff, 0xe3, 0xe3,
  0xe1, 0xfb, 0x0f, 0x00, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xf7, 0xff, 0x3e,
  0xff, 0xff, 0xe3, 0xe3, 0xc3, 0xef, 0xf8, 0x00, 0xff, 0xff, 0xe3, 0xe3,
  0xa3, 0xdf, 0xf8, 0xf8, 0xff, 0xff, 0xe3, 0xe3, 0x43, 0xbf, 0xff, 0xff,
  0xff, 0xff, 0xe3, 0xe3, 0xa3, 0x7f, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xe3,
  0xe3, 0xfd, 0xfe, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xfa, 0x0d, 0x0f,
  0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xf5, 0x0b, 0x00, 0xff, 0xff, 0xe3, 0xe3,
  0xe3, 0xeb, 0xf7, 0x3e, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xd7, 0xe8, 0x00,
  0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xaf, 0xd8, 0xf8, 0xff, 0xff, 0xe3, 0xe3,
  0xe3, 0x5f, 0xbf, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xbf, 0x7f, 0xff,
  0x01, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x05, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x04, 0x0a, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x08, 0x14, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x28, 0x00, 0x00,
  0x04, 0x00, 0x00, 0x00, 0x20, 0x50, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x40, 0xa0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x40, 0x00, 0x00,
  0x04, 0x00, 0x00, 0x00, 0x0f, 0x0d, 0x0e, 0x04, 0x04, 0x00, 0x00, 0x00,
  0x1f, 0x1a, 0x1d, 0x1c, 0x1c, 0x00, 0x00, 0x00, 0x7f, 0x75, 0x7b, 0x7c,
  0x7c, 0x00, 0x00, 0x00, 0xff, 0xeb, 0xf7, 0xfc, 0xfc, 0x00, 0x00, 0x00,
  0xff, 0xd7, 0xef, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xaf, 0xdf, 0xfc,
  0xfc, 0xfc, 0xff, 0xff, 0xff, 0x5f, 0xbf, 0xfc, 0xfc, 0xfc, 0xff, 0xff,
  0xff, 0xbf, 0x7f, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0x1f, 0x1f, 0x1d, 0x1c,
  0x0c, 0x04, 0x00, 0x00, 0x7f, 0x7f, 0x7a, 0x7c, 0x7c, 0x3c, 0x00, 0x00,
  0xff, 0xff, 0xf5, 0xf8, 0xfc, 0xfc, 0x00, 0x00, 0xff, 0xff, 0xeb, 0xf4,
  0xfc, 0xfc, 0x00, 0x00, 0xff, 0xff, 0xd7, 0xec, 0xfc, 0xfc, 0xff, 0xff,
  0xff, 0xff, 0xaf, 0xdc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0x5f, 0xbc,
  0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xbf, 0x7c, 0xfc, 0xfc, 0xff, 0xff,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xf8,
  0xfc, 0xfc, 0x7f, 0x07, 0xff, 0xff, 0xff, 0xf4, 0xf8, 0xfc, 0xfe, 0x00,
  0xff, 0xff, 0xff, 0xe8, 0xf4, 0xfc, 0xfe, 0xfe, 0xff, 0xff, 0xff, 0xd4,
  0xec, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xac, 0xdc, 0xfc, 0xff, 0xff,
  0xff, 0xff, 0xff, 0x5c, 0xbc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xbc,
  0x7c, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0x5f,
  0xff, 0xff, 0xff, 0xfc, 0xf8, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc,
  0xf4, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xe8, 0xf4, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xfc, 0xd4, 0xec, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc,
  0xac, 0xdc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0x5c, 0xbc, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xfc, 0xbc, 0x7c, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc,
  0xfc, 0xfc, 0xfe, 0xdf, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xf8, 0xfd, 0xff,
  0xff, 0xff, 0xff, 0xfc, 0xfc, 0xf4, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xfc,
  0xfc, 0xe8, 0xf7, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xd4, 0xef, 0xff,
  0xff, 0xff, 0xff, 0xfc, 0xfc, 0xac, 0xdf, 0xff, 0xff, 0xff, 0xff, 0xfc,
  0xfc, 0x5c, 0xbf, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xbc, 0x7f, 0xff,
  0x01, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x05, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x04, 0x0a, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x08, 0x14, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x28, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x20, 0x50, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x40, 0xa0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x40, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x0f, 0x0d, 0x0e, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x1f, 0x1a, 0x1d, 0x08, 0x08, 0x00, 0x00, 0x00, 0x3f, 0x35, 0x3b, 0x38,
  0x38, 0x00, 0x00, 0x00, 0xff, 0xeb, 0xf7, 0xf8, 0xf8, 0x00, 0x00, 0x00,
  0xff, 0xd7, 0xef, 0xf8, 0xf8, 0x00, 0x00, 0x00, 0xff, 0xaf, 0xdf, 0xf8,
  0xf8, 0xf8, 0xff, 0xff, 0xff, 0x5f, 0xbf, 0xf8, 0xf8, 0xf8, 0xff, 0xff,
  0xff, 0xbf, 0x7f, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0x1f, 0x1f, 0x1d, 0x18,
  0x08, 0x00, 0x00, 0x00, 0x3f, 0x3f, 0x3a, 0x38, 0x18, 0x00, 0x00, 0x00,
  0xff, 0xff, 0xf5, 0xf8, 0x78, 0x78, 0x00, 0x00, 0xff, 0xff, 0xeb, 0xf0,
  0xf8, 0xf8, 0x00, 0x00, 0xff, 0xff, 0xd7, 0xe8, 0xf8, 0xf8, 0x00, 0x00,
  0xff, 0xff, 0xaf, 0xd8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0x5f, 0xb8,
  0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xbf, 0x78, 0xf8, 0xf8, 0xff, 0xff,
  0x3f, 0x3f, 0x3f, 0x38, 0x38, 0x18, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xf0, 0xf8, 0xf8, 0x7e, 0x00,
  0xff, 0xff, 0xff, 0xe8, 0xf0, 0xf8, 0xfc, 0x00, 0xff, 0xff, 0xff, 0xd0,
  0xe8, 0xf8, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xa8, 0xd8, 0xf8, 0xff, 0xff,
  0xff, 0xff, 0xff, 0x58, 0xb8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xb8,
  0x78, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0x3f, 0x00,
  0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0x1f, 0xff, 0xff, 0xff, 0xf8,
  0xf0, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xe8, 0xf0, 0xfe, 0xfe,
  0xff, 0xff, 0xff, 0xf8, 0xd0, 0xe8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8,
  0xa8, 0xd8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0x58, 0xb8, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xf8, 0xb8, 0x78, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8,
  0xf8, 0x78, 0x3e, 0x00, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xfd, 0x5f,
  0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf0, 0xfb, 0xff, 0xff, 0xff, 0xff, 0xf8,
  0xf8, 0xe8, 0xf6, 0xfe, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xd0, 0xef, 0xff,
  0xff, 0xff, 0xff, 0xf8, 0xf8, 0xa8, 0xdf, 0xff, 0xff, 0xff, 0xff, 0xf8,
  0xf8, 0x58, 0xbf, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xb8, 0x7f, 0xff,
  0x01, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x05, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x04, 0x0a, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x08, 0x14, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x28, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x20, 0x50, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x40, 0xa0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x40, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x1f, 0x1d, 0x1e, 0x01, 0x01, 0x00, 0x00, 0x00,
  0x1f, 0x1a, 0x1d, 0x01, 0x01, 0x00, 0x00, 0x00, 0x3f, 0x35, 0x3b, 0x11,
  0x11, 0x00, 0x00, 0x00, 0x7f, 0x6b, 0x77, 0x70, 0x70, 0x00, 0x00, 0x00,
  0xff, 0xd7, 0xef, 0xf0, 0xf0, 0x00, 0x00, 0x00, 0xfe, 0xae, 0xde, 0xf0,
  0xf0, 0x00, 0x00, 0x00, 0xff, 0x5f, 0xbf, 0xf1, 0xf1, 0xf1, 0xff, 0xff,
  0xff, 0xbf, 0x7f, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0x3f, 0x3f, 0x3d, 0x30,
  0x01, 0x01, 0x00, 0x00, 0x3f, 0x3f, 0x3a, 0x31, 0x01, 0x01, 0x00, 0x00,
  0x7f, 0x7f, 0x75, 0x71, 0x31, 0x00, 0x00, 0x00, 0xff, 0xff, 0xeb, 0xf1,
  0xf0, 0xf0, 0x00, 0x00, 0xff, 0xff, 0xd7, 0xe1, 0xf0, 0xf0, 0x00, 0x00,
  0xff, 0xff, 0xaf, 0xd1, 0xf0, 0xf0, 0x00, 0x00, 0xff, 0xff, 0x5f, 0xb1,
  0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xbf, 0x71, 0xf1, 0xf1, 0xff, 0xff,
  0x7f, 0x7f, 0x7f, 0x71, 0x70, 0x01, 0x03, 0x00, 0x7f, 0x7f, 0x7f, 0x70,
  0x71, 0x01, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0xff, 0xff, 0xff, 0xe1, 0xf1, 0xf0, 0xfc, 0x00, 0xff, 0xff, 0xff, 0xd1,
  0xe1, 0xf0, 0xf8, 0x00, 0xff, 0xff, 0xff, 0xa1, 0xd1, 0xf1, 0xf8, 0xf8,
  0xff, 0xff, 0xff, 0x51, 0xb1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xff, 0xb1,
  0x71, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf0, 0x0f, 0x07,
  0xff, 0xff, 0xff, 0xf1, 0xf0, 0xf1, 0x0f, 0x0f, 0xff, 0xff, 0xff, 0xf1,
  0xf1, 0xf1, 0xff, 0x1f, 0xff, 0xff, 0xff, 0xf1, 0xe1, 0xf1, 0xfe, 0xfe,
  0xff, 0xff, 0xff, 0xf1, 0xd1, 0xe1, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xf1,
  0xa1, 0xd1, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf1, 0x51, 0xb1, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xf1, 0xb1, 0x71, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf1,
  0xf1, 0xf1, 0x0e, 0x07, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf0, 0x0d, 0x0f,
  0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xfb, 0x3f, 0xff, 0xff, 0xff, 0xf1,
  0xf1, 0xe1, 0xf6, 0xfe, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xd1, 0xec, 0xfc,
  0xff, 0xff, 0xff, 0xf1, 0xf1, 0xa1, 0xdf, 0xff, 0xff, 0xff, 0xff, 0xf1,
  0xf1, 0x51, 0xbf, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xb1, 0x7f, 0xff,
  0x01, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x05, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x04, 0x0a, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x08, 0x14, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x28, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x20, 0x50, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x40, 0xa0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x40, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x1f, 0x1d, 0x1e, 0x03, 0x03, 0x02, 0x00, 0x00,
  0x3f, 0x3a, 0x3d, 0x03, 0x03, 0x00, 0x00, 0x00, 0x3f, 0x35, 0x3b, 0x03,
  0x03, 0x00, 0x00, 0x00, 0x7f, 0x6b, 0x77, 0x22, 0x22, 0x00, 0x00, 0x00,
  0xfe, 0xd6, 0xee, 0xe0, 0xe0, 0x00, 0x00, 0x00, 0xfe, 0xae, 0xde, 0xe0,
  0xe0, 0x00, 0x00, 0x00, 0xfc, 0x5c, 0xbc, 0xe0, 0xe0, 0x00, 0x00, 0x00,
  0xff, 0xbf, 0x7f, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0x3f, 0x3f, 0x3d, 0x22,
  0x03, 0x03, 0x02, 0x00, 0x7f, 0x7f, 0x7a, 0x61, 0x03, 0x03, 0x00, 0x00,
  0x7f, 0x7f, 0x75, 0x63, 0x03, 0x03, 0x00, 0x00, 0xff, 0xff, 0xeb, 0xe3,
  0x63, 0x00, 0x00, 0x00, 0xff, 0xff, 0xd7, 0xe3, 0xe0, 0xe0, 0x00, 0x00,
  0xff, 0xff, 0xaf, 0xc3, 0xe0, 0xe0, 0x00, 0x00, 0xfe, 0xfe, 0x5e, 0xa2,
  0xe0, 0xe0, 0x00, 0x00, 0xff, 0xff, 0xbf, 0x63, 0xe3, 0xe3, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xe1, 0xe2, 0xe3, 0x0f, 0x07, 0xff, 0xff, 0xff, 0xe2,
  0xe1, 0x03, 0x07, 0x00, 0xff, 0xff, 0xff, 0xe1, 0xe3, 0x03, 0x07, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xc3,
  0xe3, 0xe0, 0xf0, 0x00, 0xff, 0xff, 0xff, 0xa3, 0xc3, 0xe0, 0xf0, 0x00,
  0xff, 0xff, 0xff, 0x43, 0xa3, 0xe3, 0xf0, 0xf0, 0xff, 0xff, 0xff, 0xa3,
  0x63, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xe1, 0xe2, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xe3, 0xe2, 0xe1, 0x0f, 0x0f, 0xff, 0xff, 0xff, 0xe3,
  0xe1, 0xe3, 0x1f, 0x1f, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0x3e,
  0xff, 0xff, 0xff, 0xe3, 0xc3, 0xe3, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xe3,
  0xa3, 0xc3, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xe3, 0x43, 0xa3, 0xff, 0xff,
  0xff, 0xff, 0xff, 0xe3, 0xa3, 0x63, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe3,
  0xe3, 0xe1, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe2, 0x0d, 0x0f,
  0xff, 0xff, 0xff, 0xe3, 0xe3, 0xe1, 0x1b, 0x1f, 0xff, 0xff, 0xff, 0xe3,
  0xe3, 0xe3, 0xf7, 0x7f, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xc3, 0xec, 0xfc,
  0xff, 0xff, 0xff, 0xe3, 0xe3, 0xa3, 0xd8, 0xf8, 0xff, 0xff, 0xff, 0xe3,
  0xe3, 0x43, 0xbf, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xa3, 0x7f, 0xff,
  0x01, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x05, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x04, 0x0a, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x08, 0x14, 0x00, 0x00, 0x00, 0x04, 0x00, 0x00, 0x10, 0x28, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x20, 0x50, 0x00, 0x00, 0x00, 0x04, 0x00, 0x00,
  0x40, 0xa0, 0x00, 0x00, 0x00, 0x04, 0x00, 0x00, 0x80, 0x40, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x3f, 0x3d, 0x3e, 0x3f, 0x1c, 0x04, 0x00, 0x00,
  0x7f, 0x7a, 0x7d, 0x7f, 0x3c, 0x1c, 0x00, 0x00, 0xff, 0xf5, 0xfb, 0xff,
  0xfc, 0xfc, 0x00, 0x00, 0xff, 0xeb, 0xf7, 0xff, 0xfc, 0xfc, 0xfc, 0xff,
  0xff, 0xd7, 0xef, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xaf, 0xdf, 0xff,
  0xfc, 0xfc, 0xfc, 0xff, 0xff, 0x5f, 0xbf, 0xff, 0xfc, 0xfc, 0xfc, 0xff,
  0xff, 0xbf, 0x7f, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0x7f, 0x7f, 0x7d, 0x7e,
  0x3c, 0x1c, 0x00, 0x00, 0xff, 0xff, 0xfa, 0x7d, 0x7c, 0x3c, 0x00, 0x00,
  0xff, 0xff, 0xf5, 0xfb, 0xfc, 0xfc, 0xfc, 0x00, 0xff, 0xff, 0xeb, 0xf7,
  0xfc, 0xfc, 0xfc, 0x00, 0xff, 0xff, 0xd7, 0xef, 0xfc, 0xfc, 0xfc, 0xff,
  0xff, 0xff, 0xaf, 0xdf, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0x5f, 0xbf,
  0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xbf, 0x7f, 0xfc, 0xfc, 0xfc, 0xff,
  0xff, 0xff, 0xff, 0xfd, 0xfc, 0xfc, 0x3c, 0x00, 0xff, 0xff, 0xff, 0xfa,
  0xfc, 0xfc, 0xfc, 0x7f, 0xff, 0xff, 0xff, 0xf5, 0xf8, 0xfc, 0xfc, 0xff,
  0xff, 0xff, 0xff, 0xeb, 0xf4, 0xfc, 0xfc, 0xfe, 0xff, 0xff, 0xff, 0xd7,
  0xec, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xaf, 0xdc, 0xfc, 0xfc, 0xff,
  0xff, 0xff, 0xff, 0x5f, 0xbc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xbf,
  0x7c, 0xfc, 0xfc, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0xff, 0xff, 0xff, 0xff, 0xf8, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xf4, 0xf8, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe8, 0xf4, 0xfc, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xd4, 0xec, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xac, 0xdc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0x5c, 0xbc, 0xfc, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xbc, 0x7c, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xf8, 0xfc, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xfc, 0xf4, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfc, 0xe8, 0xf4, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xd4, 0xec, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xfc, 0xac, 0xdc, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xfc, 0x5c, 0xbc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xbc, 0x7c, 0xff,
  0x01, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x05, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x04, 0x0a, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x08, 0x14, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x28, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x20, 0x50, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x40, 0xa0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x40, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x3f, 0x3d, 0x3e, 0x3f, 0x08, 0x00, 0x00, 0x00,
  0x7f, 0x7a, 0x7d, 0x7f, 0x38, 0x08, 0x00, 0x00, 0xff, 0xf5, 0xfb, 0xff,
  0x38, 0x38, 0x00, 0x00, 0xff, 0xeb, 0xf7, 0xff, 0xf8, 0xf8, 0x00, 0x00,
  0xff, 0xd7, 0xef, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xaf, 0xdf, 0xff,
  0xf8, 0xf8, 0xf8, 0xff, 0xff, 0x5f, 0xbf, 0xff, 0xf8, 0xf8, 0xf8, 0xff,
  0xff, 0xbf, 0x7f, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0x7f, 0x7f, 0x7d, 0x3e,
  0x38, 0x08, 0x00, 0x00, 0x7f, 0x7f, 0x7a, 0x7d, 0x78, 0x08, 0x00, 0x00,
  0xff, 0xff, 0xf5, 0xfb, 0xf8, 0x78, 0x00, 0x00, 0xff, 0xff, 0xeb, 0xf7,
  0xf8, 0xf8, 0xf8, 0x00, 0xff, 0xff, 0xd7, 0xef, 0xf8, 0xf8, 0xf8, 0x00,
  0xff, 0xff, 0xaf, 0xdf, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0x5f, 0xbf,
  0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xbf, 0x7f, 0xf8, 0xf8, 0xf8, 0xff,
  0xff, 0xff, 0xff, 0xfd, 0x78, 0x78, 0x08, 0x00, 0xff, 0xff, 0xff, 0xfa,
  0xf8, 0xf8, 0x08, 0x07, 0xff, 0xff, 0xff, 0xf5, 0xf8, 0xf8, 0xf8, 0xfe,
  0xff, 0xff, 0xff, 0xeb, 0xf0, 0xf8, 0xf8, 0xfc, 0xff, 0xff, 0xff, 0xd7,
  0xe8, 0xf8, 0xf8, 0xfc, 0xff, 0xff, 0xff, 0xaf, 0xd8, 0xf8, 0xf8, 0xff,
  0xff, 0xff, 0xff, 0x5f, 0xb8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xbf,
  0x78, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0x5f,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff,
  0xf0, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe8, 0xf0, 0xf8, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xd0, 0xe8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xa8, 0xd8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0x58, 0xb8, 0xf8, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xb8, 0x78, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xf8, 0xf0, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xf8, 0xe8, 0xf0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xd0, 0xe8, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xf8, 0xa8, 0xd8, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xf8, 0x58, 0xb8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xb8, 0x78, 0xff,
  0x01, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x05, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x04, 0x0a, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x08, 0x14, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x28, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x20, 0x50, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x40, 0xa0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x40, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x3f, 0x3d, 0x3e, 0x3f, 0x01, 0x01, 0x00, 0x00,
  0x7f, 0x7a, 0x3d, 0x7f, 0x01, 0x01, 0x00, 0x00, 0xff, 0xf5, 0xfb, 0xff,
  0x71, 0x11, 0x00, 0x00, 0xff, 0xeb, 0xf7, 0xff, 0xf1, 0x70, 0x00, 0x00,
  0xff, 0xd7, 0xef, 0xff, 0xf0, 0xf0, 0x00, 0x00, 0xff, 0xaf, 0xdf, 0xff,
  0xf1, 0xf1, 0xf1, 0xff, 0xff, 0x5f, 0xbf, 0xff, 0xf1, 0xf1, 0xf1, 0xff,
  0xff, 0xbf, 0x7f, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0x7f, 0x7f, 0x7d, 0x3e,
  0x31, 0x01, 0x00, 0x00, 0x7f, 0x7f, 0x7a, 0x7d, 0x71, 0x01, 0x00, 0x00,
  0xff, 0xff, 0xf5, 0xfb, 0xf1, 0x11, 0x00, 0x00, 0xff, 0xff, 0xeb, 0xf7,
  0xf1, 0xf0, 0x00, 0x00, 0xff, 0xff, 0xd7, 0xef, 0xf1, 0xf0, 0xf0, 0x00,
  0xff, 0xff, 0xaf, 0xdf, 0xf1, 0xf0, 0xf0, 0x00, 0xff, 0xff, 0x5f, 0xbf,
  0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xbf, 0x7f, 0xf1, 0xf1, 0xf1, 0xff,
  0xff, 0xff, 0xff, 0xfd, 0x70, 0x71, 0x01, 0x00, 0xff, 0xff, 0xff, 0xfa,
  0xf1, 0xf1, 0x01, 0x07, 0xff, 0xff, 0xff, 0xf5, 0xf1, 0xf1, 0x11, 0x0e,
  0xff, 0xff, 0xff, 0xeb, 0xf1, 0xf1, 0xf0, 0xfc, 0xff, 0xff, 0xff, 0xd7,
  0xe1, 0xf1, 0xf0, 0xf8, 0xff, 0xff, 0xff, 0xaf, 0xd1, 0xf1, 0xf0, 0xf8,
  0xff, 0xff, 0xff, 0x5f, 0xb1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xbf,
  0x71, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xf0, 0xf1, 0x1f,
  0xff, 0xff, 0xff, 0xff, 0xf0, 0xf1, 0xf1, 0x1f, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0xe1, 0xf1, 0xf1, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xd1, 0xe1, 0xf1, 0xfe, 0xff, 0xff, 0xff, 0xff,
  0xa1, 0xd1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xff, 0x51, 0xb1, 0xf1, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xb1, 0x71, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xf1, 0xf1, 0xf0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xf0, 0xf1, 0xdf,
  0xff, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xf1, 0xe1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xd1, 0xe1, 0xfe,
  0xff, 0xff, 0xff, 0xff, 0xf1, 0xa1, 0xd1, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xf1, 0x51, 0xb1, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf1, 0xb1, 0x71, 0xff,
  0x01, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x05, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x04, 0x0a, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x08, 0x14, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x28, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x20, 0x50, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x40, 0xa0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x40, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0xff, 0xfd, 0xbe, 0xbf, 0x83, 0xe3, 0xe2, 0xff,
  0x7f, 0x7a, 0x7d, 0x7f, 0x03, 0x03, 0x00, 0x00, 0xff, 0xf5, 0xfb, 0xff,
  0x23, 0x03, 0x00, 0x00, 0xff, 0xeb, 0xf7, 0xff, 0xe3, 0x22, 0x00, 0x00,
  0xff, 0xd7, 0xef, 0xff, 0xe2, 0xe0, 0x00, 0x00, 0xff, 0xaf, 0xdf, 0xff,
  0xe0, 0xe0, 0x00, 0x00, 0xff, 0x5f, 0xbf, 0xff, 0xe3, 0xe3, 0xe3, 0xff,
  0xff, 0xbf, 0x7f, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0x7f, 0x7f, 0x7d, 0x7e,
  0x63, 0x03, 0x03, 0x00, 0xff, 0xff, 0xfa, 0x7d, 0x63, 0x03, 0x03, 0x00,
  0xff, 0xff, 0xf5, 0xfb, 0xe3, 0x03, 0x00, 0x00, 0xff, 0xff, 0xeb, 0xf7,
  0xe3, 0x22, 0x00, 0x00, 0xff, 0xff, 0xd7, 0xef, 0xe3, 0xe0, 0x00, 0x00,
  0xff, 0xff, 0xaf, 0xdf, 0xe3, 0xe0, 0xe0, 0x00, 0xff, 0xff, 0x5f, 0xbf,
  0xe3, 0xe0, 0xe0, 0x00, 0xff, 0xff, 0xbf, 0x7f, 0xe3, 0xe3, 0xe3, 0xff,
  0xff, 0xff, 0xff, 0xfd, 0xe2, 0xe3, 0x03, 0x07, 0xff, 0xff, 0xff, 0xfa,
  0xe1, 0xe3, 0x03, 0x07, 0xff, 0xff, 0xff, 0xf5, 0xe3, 0xe3, 0x03, 0x0f,
  0xff, 0xff, 0xff, 0xeb, 0xe3, 0xe3, 0x22, 0x1c, 0xff, 0xff, 0xff, 0xd7,
  0xe3, 0xe3, 0xe0, 0xf8, 0xff, 0xff, 0xff, 0xaf, 0xc3, 0xe3, 0xe0, 0xf0,
  0xff, 0xff, 0xff, 0x5f, 0xa3, 0xe3, 0xe0, 0xf0, 0xff, 0xff, 0xff, 0xbf,
  0x63, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe1, 0xe2, 0xe3, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xe2, 0xe1, 0xe3, 0x1f, 0xff, 0xff, 0xff, 0xff,
  0xe1, 0xe3, 0xe3, 0x3f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0xff, 0xff, 0xff, 0xff, 0xc3, 0xe3, 0xe3, 0xfe, 0xff, 0xff, 0xff, 0xff,
  0xa3, 0xc3, 0xe3, 0xfc, 0xff, 0xff, 0xff, 0xff, 0x43, 0xa3, 0xe3, 0xff,
  0xff, 0xff, 0xff, 0xff, 0xa3, 0x63, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xff,
  0xe3, 0xe1, 0xe2, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xe2, 0xe1, 0x1f,
  0xff, 0xff, 0xff, 0xff, 0xe3, 0xe1, 0xe3, 0x3f, 0xff, 0xff, 0xff, 0xff,
  0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xc3, 0xe3, 0xfe,
  0xff, 0xff, 0xff, 0xff, 0xe3, 0xa3, 0xc3, 0xfc, 0xff, 0xff, 0xff, 0xff,
  0xe3, 0x43, 0xa3, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe3, 0xa3, 0x63, 0xff,
  0x01, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x05, 0x00, 0x00,
  0x00, 0x00, 0x04, 0x00, 0x04, 0x0a, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x08, 0x14, 0x00, 0x00, 0x00, 0x00, 0x04, 0x00, 0x10, 0x28, 0x00, 0x00,
  0x00, 0x00, 0x04, 0x00, 0x20, 0x50, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x40, 0xa0, 0x00, 0x00, 0x00, 0x00, 0x04, 0x00, 0x80, 0x40, 0x00, 0x00,
  0x00, 0x00, 0x04, 0x00, 0xff, 0xfd, 0xfe, 0xff, 0xff, 0xfc, 0xfc, 0xfc,
  0xff, 0xfa, 0xfd, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xf5, 0xfb, 0xff,
  0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xeb, 0xf7, 0xff, 0xff, 0xfc, 0xfc, 0xfc,
  0xff, 0xd7, 0xef, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xaf, 0xdf, 0xff,
  0xff, 0xfc, 0xfc, 0xfc, 0xff, 0x5f, 0xbf, 0xff, 0xff, 0xfc, 0xfc, 0xfc,
  0xff, 0xbf, 0x7f, 0xff, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xfd, 0xfe,
  0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xfa, 0xfd, 0xff, 0xfc, 0xfc, 0xfc,
  0xff, 0xff, 0xf5, 0xfb, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xeb, 0xf7,
  0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xd7, 0xef, 0xff, 0xfc, 0xfc, 0xfc,
  0xff, 0xff, 0xaf, 0xdf, 0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0x5f, 0xbf,
  0xff, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xbf, 0x7f, 0xff, 0xfc, 0xfc, 0xfc,
  0xff, 0xff, 0xff, 0xfd, 0xfe, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xfa,
  0xfd, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xf5, 0xfb, 0xfc, 0xfc, 0xfc,
  0xff, 0xff, 0xff, 0xeb, 0xf7, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xd7,
  0xef, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xaf, 0xdf, 0xfc, 0xfc, 0xfc,
  0xff, 0xff, 0xff, 0x5f, 0xbf, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xbf,
  0x7f, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xfc, 0xfc, 0xfc,
  0xff, 0xff, 0xff, 0xff, 0xfa, 0xfc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff,
  0xf5, 0xf8, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xeb, 0xf4, 0xfc, 0xfc,
  0xff, 0xff, 0xff, 0xff, 0xd7, 0xec, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff,
  0xaf, 0xdc, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0x5f, 0xbc, 0xfc, 0xfc,
  0xff, 0xff, 0xff, 0xff, 0xbf, 0x7c, 0xfc, 0xfc, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xfc, 0xfc,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xf4, 0xf8, 0xfc, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xe8, 0xf4, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xd4, 0xec, 0xfc,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xac, 0xdc, 0xfc, 0xff, 0xff, 0xff, 0xff,
  0xff, 0x5c, 0xbc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xbc, 0x7c, 0xfc,
  0x01, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x05, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x04, 0x0a, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x08, 0x14, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x28, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x20, 0x50, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x40, 0xa0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x40, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0xff, 0xfd, 0xfe, 0xff, 0xff, 0xf8, 0xf8, 0xf8,
  0xff, 0xfa, 0xfd, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xf5, 0xfb, 0xff,
  0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xeb, 0xf7, 0xff, 0xff, 0xf8, 0xf8, 0xf8,
  0xff, 0xd7, 0xef, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xaf, 0xdf, 0xff,
  0xff, 0xf8, 0xf8, 0xf8, 0xff, 0x5f, 0xbf, 0xff, 0xff, 0xf8, 0xf8, 0xf8,
  0xff, 0xbf, 0x7f, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xfd, 0xfe,
  0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xfa, 0xfd, 0xff, 0xf8, 0xf8, 0xf8,
  0xff, 0xff, 0xf5, 0xfb, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xeb, 0xf7,
  0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xd7, 0xef, 0xff, 0xf8, 0xf8, 0xf8,
  0xff, 0xff, 0xaf, 0xdf, 0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0x5f, 0xbf,
  0xff, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xbf, 0x7f, 0xff, 0xf8, 0xf8, 0xf8,
  0xff, 0xff, 0xff, 0xfd, 0xfe, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xfa,
  0xfd, 0xf8, 0xf8, 0x78, 0xff, 0xff, 0xff, 0xf5, 0xfb, 0xf8, 0xf8, 0xf8,
  0xff, 0xff, 0xff, 0xeb, 0xf7, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xd7,
  0xef, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xaf, 0xdf, 0xf8, 0xf8, 0xf8,
  0xff, 0xff, 0xff, 0x5f, 0xbf, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xbf,
  0x7f, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xf8, 0xf8, 0xf8,
  0xff, 0xff, 0xff, 0xff, 0xfa, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff,
  0xf5, 0xf8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xeb, 0xf0, 0xf8, 0xf8,
  0xff, 0xff, 0xff, 0xff, 0xd7, 0xe8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff,
  0xaf, 0xd8, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0x5f, 0xb8, 0xf8, 0xf8,
  0xff, 0xff, 0xff, 0xff, 0xbf, 0x78, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xf8, 0xf8, 0xf8, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xf0, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xe8, 0xf0, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xd0, 0xe8, 0xf8,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xa8, 0xd8, 0xf8, 0xff, 0xff, 0xff, 0xff,
  0xff, 0x58, 0xb8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xb8, 0x78, 0xf8,
  0x01, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x05, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x04, 0x0a, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x08, 0x14, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x28, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x20, 0x50, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x40, 0xa0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x40, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0xff, 0xfd, 0xfe, 0xff, 0xff, 0xf1, 0xf1, 0xf1,
  0xff, 0xfa, 0xfd, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xf5, 0xfb, 0xff,
  0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xeb, 0xf7, 0xff, 0xff, 0xf1, 0xf1, 0xf1,
  0xff, 0xd7, 0xef, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xaf, 0xdf, 0xff,
  0xff, 0xf1, 0xf1, 0xf1, 0xff, 0x5f, 0xbf, 0xff, 0xff, 0xf1, 0xf1, 0xf1,
  0xff, 0xbf, 0x7f, 0xff, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xfd, 0xfe,
  0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xfa, 0xfd, 0xff, 0xf1, 0xf1, 0xf1,
  0xff, 0xff, 0xf5, 0xfb, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xeb, 0xf7,
  0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xd7, 0xef, 0xff, 0xf1, 0xf1, 0xf1,
  0xff, 0xff, 0xaf, 0xdf, 0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0x5f, 0xbf,
  0xff, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xbf, 0x7f, 0xff, 0xf1, 0xf1, 0xf1,
  0xff, 0xff, 0xff, 0xfd, 0xfe, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xfa,
  0xfd, 0xf1, 0xf1, 0x11, 0xff, 0xff, 0xff, 0xf5, 0xfb, 0xf1, 0xf1, 0xf1,
  0xff, 0xff, 0xff, 0xeb, 0xf7, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xd7,
  0xef, 0xf1, 0xf1, 0xf0, 0xff, 0xff, 0xff, 0xaf, 0xdf, 0xf1, 0xf1, 0xf1,
  0xff, 0xff, 0xff, 0x5f, 0xbf, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xbf,
  0x7f, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xf0, 0xf1, 0xf1,
  0xff, 0xff, 0xff, 0xff, 0xfa, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff,
  0xf5, 0xf1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xeb, 0xf1, 0xf1, 0xf1,
  0xff, 0xff, 0xff, 0xff, 0xd7, 0xe1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff,
  0xaf, 0xd1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0x5f, 0xb1, 0xf1, 0xf1,
  0xff, 0xff, 0xff, 0xff, 0xbf, 0x71, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xf1, 0xf0, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf0, 0xf1, 0xf1,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xe1, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xff, 0xd1, 0xe1, 0xf1,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xa1, 0xd1, 0xf1, 0xff, 0xff, 0xff, 0xff,
  0xff, 0x51, 0xb1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xff, 0xb1, 0x71, 0xf1,
  0x01, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x05, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x04, 0x0a, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x08, 0x14, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x28, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x20, 0x50, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x40, 0xa0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x40, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0xff, 0xfd, 0xfe, 0xff, 0xff, 0xe3, 0xe3, 0xe3,
  0xff, 0xfa, 0xfd, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xf5, 0xfb, 0xff,
  0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xeb, 0xf7, 0xff, 0xff, 0xe3, 0xe3, 0xe3,
  0xff, 0xd7, 0xef, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xaf, 0xdf, 0xff,
  0xff, 0xe3, 0xe3, 0xe3, 0xff, 0x5f, 0xbf, 0xff, 0xff, 0xe3, 0xe3, 0xe3,
  0xff, 0xbf, 0x7f, 0xff, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xfd, 0xfe,
  0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xfa, 0xfd, 0xff, 0xe3, 0xe3, 0xe3,
  0xff, 0xff, 0xf5, 0xfb, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xeb, 0xf7,
  0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xd7, 0xef, 0xff, 0xe3, 0xe3, 0xe3,
  0xff, 0xff, 0xaf, 0xdf, 0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0x5f, 0xbf,
  0xff, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xbf, 0x7f, 0xff, 0xe3, 0xe3, 0xe3,
  0xff, 0xff, 0xff, 0xfd, 0xfe, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xfa,
  0xfd, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xf5, 0xfb, 0xe3, 0xe3, 0x23,
  0xff, 0xff, 0xff, 0xeb, 0xf7, 0xe3, 0xe3, 0x63, 0xff, 0xff, 0xff, 0xd7,
  0xef, 0xe3, 0xe3, 0xe2, 0xff, 0xff, 0xff, 0xaf, 0xdf, 0xe3, 0xe3, 0xe0,
  0xff, 0xff, 0xff, 0x5f, 0xbf, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xbf,
  0x7f, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xe2, 0xe3, 0xe3,
  0xff, 0xff, 0xff, 0xff, 0xfa, 0xe1, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff,
  0xf5, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xeb, 0xe3, 0xe3, 0xe3,
  0xff, 0xff, 0xff, 0xff, 0xd7, 0xe3, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff,
  0xaf, 0xc3, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff, 0x5f, 0xa3, 0xe3, 0xe3,
  0xff, 0xff, 0xff, 0xff, 0xbf, 0x63, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xe1, 0xe2, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe2, 0xe1, 0xe3,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xe1, 0xe3, 0xe3, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xc3, 0xe3, 0xe3,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xa3, 0xc3, 0xe3, 0xff, 0xff, 0xff, 0xff,
  0xff, 0x43, 0xa3, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xff, 0xa3, 0x63, 0xe3,
  0x01, 0x02, 0x00, 0x00, 0x00, 0x02, 0x00, 0x00, 0x02, 0x05, 0x00, 0x00,
  0x00, 0x00, 0x04, 0x04, 0x04, 0x0a, 0x00, 0x00, 0x00, 0x03, 0x00, 0x00,
  0x08, 0x14, 0x00, 0x00, 0x00, 0x03, 0x04, 0x04, 0x10, 0x28, 0x00, 0x00,
  0x00, 0x03, 0x04, 0x04, 0x20, 0x50, 0x00, 0x00, 0x00, 0x02, 0x04, 0x04,
  0x40, 0xa0, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x80, 0x40, 0x00, 0x00,
  0x00, 0x03, 0x04, 0x04, 0xff, 0xfd, 0xfe, 0xff, 0xff, 0xff, 0xfc, 0xfc,
  0xff, 0xfa, 0xfd, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xff, 0xf5, 0xfb, 0xff,
  0xff, 0xff, 0xfc, 0xfc, 0xff, 0xeb, 0xf7, 0xff, 0xff, 0xff, 0xfc, 0xfc,
  0xff, 0xd7, 0xef, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xff, 0xaf, 0xdf, 0xff,
  0xff, 0xff, 0xfc, 0xfc, 0xff, 0x5f, 0xbf, 0xff, 0xff, 0xff, 0xfc, 0xfc,
  0xff, 0xbf, 0x7f, 0xff, 0xff, 0xff, 0xfc, 0xfc, 0xff, 0xff, 0xfd, 0xfe,
  0xff, 0xff, 0xfc, 0xfc, 0xff, 0xff, 0xfa, 0xfd, 0xff, 0xff, 0xfc, 0xfc,
  0xff, 0xff, 0xf5, 0xfb, 0xff, 0xff, 0xfc, 0xfc, 0xff, 0xff, 0xeb, 0xf7,
  0xff, 0xff, 0xfc, 0xfc, 0xff, 0xff, 0xd7, 0xef, 0xff, 0xff, 0xfc, 0xfc,
  0xff, 0xff, 0xaf, 0xdf, 0xff, 0xff, 0xfc, 0xfc, 0xff, 0xff, 0x5f, 0xbf,
  0xff, 0xff, 0xfc, 0xfc, 0xff, 0xff, 0xbf, 0x7f, 0xff, 0xff, 0xfc, 0xfc,
  0xff, 0xff, 0xff, 0xfd, 0xfe, 0xff, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xfa,
  0xfd, 0xff, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xf5, 0xfb, 0xff, 0xfc, 0xfc,
  0xff, 0xff, 0xff, 0xeb, 0xf7, 0xff, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xd7,
  0xef, 0xff, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xaf, 0xdf, 0xff, 0xfc, 0xfc,
  0xff, 0xff, 0xff, 0x5f, 0xbf, 0xff, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xbf,
  0x7f, 0xff, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xfe, 0xfc, 0xfc,
  0xff, 0xff, 0xff, 0xff, 0xfa, 0xfd, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff,
  0xf5, 0xfb, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xeb, 0xf7, 0xfc, 0xfc,
  0xff, 0xff, 0xff, 0xff, 0xd7, 0xef, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff,
  0xaf, 0xdf, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0x5f, 0xbf, 0xfc, 0xfc,
  0xff, 0xff, 0xff, 0xff, 0xbf, 0x7f, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xfd, 0xfc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfa, 0xfc, 0xfc,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xf5, 0xf8, 0xfc, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xeb, 0xf4, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xd7, 0xec, 0xfc,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xaf, 0xdc, 0xfc, 0xff, 0xff, 0xff, 0xff,
  0xff, 0x5f, 0xbc, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xbf, 0x7c, 0xfc,
  0x01, 0x02, 0x00, 0x00, 0x00, 0x02, 0x00, 0x00, 0x02, 0x05, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x04, 0x0a, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x08, 0x14, 0x00, 0x00, 0x00, 0x02, 0x00, 0x00, 0x10, 0x28, 0x00, 0x00,
  0x00, 0x02, 0x00, 0x00, 0x20, 0x50, 0x00, 0x00, 0x00, 0x02, 0x00, 0x00,
  0x40, 0xa0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x40, 0x00, 0x00,
  0x00, 0x02, 0x00, 0x00, 0xff, 0xfd, 0xfe, 0xff, 0xff, 0xff, 0xf8, 0xf8,
  0xff, 0xfa, 0xfd, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xff, 0xf5, 0xfb, 0xff,
  0xff, 0xff, 0xf8, 0xf8, 0xff, 0xeb, 0xf7, 0xff, 0xff, 0xff, 0xf8, 0xf8,
  0xff, 0xd7, 0xef, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xff, 0xaf, 0xdf, 0xff,
  0xff, 0xff, 0xf8, 0xf8, 0xff, 0x5f, 0xbf, 0xff, 0xff, 0xff, 0xf8, 0xf8,
  0xff, 0xbf, 0x7f, 0xff, 0xff, 0xff, 0xf8, 0xf8, 0xff, 0xff, 0xfd, 0xfe,
  0xff, 0xff, 0xf8, 0xf8, 0xff, 0xff, 0xfa, 0xfd, 0xff, 0xff, 0xf8, 0xf8,
  0xff, 0xff, 0xf5, 0xfb, 0xff, 0xff, 0xf8, 0xf8, 0xff, 0xff, 0xeb, 0xf7,
  0xff, 0xff, 0xf8, 0xf8, 0xff, 0xff, 0xd7, 0xef, 0xff, 0xff, 0xf8, 0xf8,
  0xff, 0xff, 0xaf, 0xdf, 0xff, 0xff, 0xf8, 0xf8, 0xff, 0xff, 0x5f, 0xbf,
  0xff, 0xff, 0xf8, 0xf8, 0xff, 0xff, 0xbf, 0x7f, 0xff, 0xff, 0xf8, 0xf8,
  0xff, 0xff, 0xff, 0xfd, 0xfe, 0xff, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xfa,
  0xfd, 0xff, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xf5, 0xfb, 0xff, 0xf8, 0xf8,
  0xff, 0xff, 0xff, 0xeb, 0xf7, 0xff, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xd7,
  0xef, 0xff, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xaf, 0xdf, 0xff, 0xf8, 0xf8,
  0xff, 0xff, 0xff, 0x5f, 0xbf, 0xff, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xbf,
  0x7f, 0xff, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xfe, 0xf8, 0xf8,
  0xff, 0xff, 0xff, 0xff, 0xfa, 0xfd, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff,
  0xf5, 0xfb, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xeb, 0xf7, 0xf8, 0xf8,
  0xff, 0xff, 0xff, 0xff, 0xd7, 0xef, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff,
  0xaf, 0xdf, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0x5f, 0xbf, 0xf8, 0xf8,
  0xff, 0xff, 0xff, 0xff, 0xbf, 0x7f, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xfd, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfa, 0xf8, 0xf8,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xf5, 0xf8, 0xf8, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xeb, 0xf0, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xd7, 0xe8, 0xf8,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xaf, 0xd8, 0xf8, 0xff, 0xff, 0xff, 0xff,
  0xff, 0x5f, 0xb8, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xff, 0xbf, 0x78, 0xf8,
  0x01, 0x02, 0x00, 0x00, 0x00, 0x04, 0x00, 0x00, 0x02, 0x05, 0x00, 0x00,
  0x00, 0x04, 0x00, 0x00, 0x04, 0x0a, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x08, 0x14, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x28, 0x00, 0x00,
  0x00, 0x04, 0x00, 0x00, 0x20, 0x50, 0x00, 0x00, 0x00, 0x04, 0x00, 0x00,
  0x40, 0xa0, 0x00, 0x00, 0x00, 0x04, 0x00, 0x00, 0x80, 0x40, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0xff, 0xfd, 0xfe, 0xff, 0xff, 0xff, 0xf1, 0xf1,
  0xff, 0xfa, 0xfd, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xff, 0xf5, 0xfb, 0xff,
  0xff, 0xff, 0xf1, 0xf1, 0xff, 0xeb, 0xf7, 0xff, 0xff, 0xff, 0xf1, 0xf1,
  0xff, 0xd7, 0xef, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xff, 0xaf, 0xdf, 0xff,
  0xff, 0xff, 0xf1, 0xf1, 0xff, 0x5f, 0xbf, 0xff, 0xff, 0xff, 0xf1, 0xf1,
  0xff, 0xbf, 0x7f, 0xff, 0xff, 0xff, 0xf1, 0xf1, 0xff, 0xff, 0xfd, 0xfe,
  0xff, 0xff, 0xf1, 0xf1, 0xff, 0xff, 0xfa, 0xfd, 0xff, 0xff, 0xf1, 0xf1,
  0xff, 0xff, 0xf5, 0xfb, 0xff, 0xff, 0xf1, 0xf1, 0xff, 0xff, 0xeb, 0xf7,
  0xff, 0xff, 0xf1, 0xf1, 0xff, 0xff, 0xd7, 0xef, 0xff, 0xff, 0xf1, 0xf1,
  0xff, 0xff, 0xaf, 0xdf, 0xff, 0xff, 0xf1, 0xf1, 0xff, 0xff, 0x5f, 0xbf,
  0xff, 0xff, 0xf1, 0xf1, 0xff, 0xff, 0xbf, 0x7f, 0xff, 0xff, 0xf1, 0xf1,
  0xff, 0xff, 0xff, 0xfd, 0xfe, 0xff, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xfa,
  0xfd, 0xff, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xf5, 0xfb, 0xff, 0xf1, 0xf1,
  0xff, 0xff, 0xff, 0xeb, 0xf7, 0xff, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xd7,
  0xef, 0xff, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xaf, 0xdf, 0xff, 0xf1, 0xf1,
  0xff, 0xff, 0xff, 0x5f, 0xbf, 0xff, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xbf,
  0x7f, 0xff, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xfe, 0xf1, 0xf1,
  0xff, 0xff, 0xff, 0xff, 0xfa, 0xfd, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff,
  0xf5, 0xfb, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xeb, 0xf7, 0xf1, 0xf1,
  0xff, 0xff, 0xff, 0xff, 0xd7, 0xef, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff,
  0xaf, 0xdf, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0x5f, 0xbf, 0xf1, 0xf1,
  0xff, 0xff, 0xff, 0xff, 0xbf, 0x7f, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xfd, 0xf0, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfa, 0xf1, 0xf1,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xf5, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xeb, 0xf1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xff, 0xd7, 0xe1, 0xf1,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xaf, 0xd1, 0xf1, 0xff, 0xff, 0xff, 0xff,
  0xff, 0x5f, 0xb1, 0xf1, 0xff, 0xff, 0xff, 0xff, 0xff, 0xbf, 0x71, 0xf1,
  0x01, 0x02, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x02, 0x05, 0x00, 0x00,
  0x00, 0x08, 0x00, 0x00, 0x04, 0x0a, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00,
  0x08, 0x14, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x28, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x20, 0x50, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00,
  0x40, 0xa0, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x80, 0x40, 0x00, 0x00,
  0x00, 0x08, 0x00, 0x00, 0xff, 0xfd, 0xfe, 0xff, 0xff, 0xff, 0xe3, 0xe3,
  0xff, 0xfa, 0xfd, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xff, 0xf5, 0xfb, 0xff,
  0xff, 0xff, 0xe3, 0xe3, 0xff, 0xeb, 0xf7, 0xff, 0xff, 0xff, 0xe3, 0xe3,
  0xff, 0xd7, 0xef, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xff, 0xaf, 0xdf, 0xff,
  0xff, 0xff, 0xe3, 0xe3, 0xff, 0x5f, 0xbf, 0xff, 0xff, 0xff, 0xe3, 0xe3,
  0xff, 0xbf, 0x7f, 0xff, 0xff, 0xff, 0xe3, 0xe3, 0xff, 0xff, 0xfd, 0xfe,
  0xff, 0xff, 0xe3, 0xe3, 0xff, 0xff, 0xfa, 0xfd, 0xff, 0xff, 0xe3, 0xe3,
  0xff, 0xff, 0xf5, 0xfb, 0xff, 0xff, 0xe3, 0xe3, 0xff, 0xff, 0xeb, 0xf7,
  0xff, 0xff, 0xe3, 0xe3, 0xff, 0xff, 0xd7, 0xef, 0xff, 0xff, 0xe3, 0xe3,
  0xff, 0xff, 0xaf, 0xdf, 0xff, 0xff, 0xe3, 0xe3, 0xff, 0xff, 0x5f, 0xbf,
  0xff, 0xff, 0xe3, 0xe3, 0xff, 0xff, 0xbf, 0x7f, 0xff, 0xff, 0xe3, 0xe3,
  0xff, 0xff, 0xff, 0xfd, 0xfe, 0xff, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xfa,
  0xfd, 0xff, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xf5, 0xfb, 0xff, 0xe3, 0xe3,
  0xff, 0xff, 0xff, 0xeb, 0xf7, 0xff, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xd7,
  0xef, 0xff, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xaf, 0xdf, 0xff, 0xe3, 0xe3,
  0xff, 0xff, 0xff, 0x5f, 0xbf, 0xff, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xbf,
  0x7f, 0xff, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xfe, 0xe3, 0xe3,
  0xff, 0xff, 0xff, 0xff, 0xfa, 0xfd, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff,
  0xf5, 0xfb, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xeb, 0xf7, 0xe3, 0xe3,
  0xff, 0xff, 0xff, 0xff, 0xd7, 0xef, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff,
  0xaf, 0xdf, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff, 0x5f, 0xbf, 0xe3, 0xe3,
  0xff, 0xff, 0xff, 0xff, 0xbf, 0x7f, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xfd, 0xe2, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfa, 0xe1, 0xe3,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xf5, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff,
  0xff, 0xeb, 0xe3, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xff, 0xd7, 0xe3, 0xe3,
  0xff, 0xff, 0xff, 0xff, 0xff, 0xaf, 0xc3, 0xe3, 0xff, 0xff, 0xff, 0xff,
  0xff, 0x5f, 0xa3, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xff, 0xbf, 0x63, 0xe3
};
